@@ -6,6 +6,7 @@ $expectedFailures = [
     'Int64 type: -1' => 'PHP encodes integers as 32-bit if range allows',
     'Int64 type: 0' => 'PHP encodes integers as 32-bit if range allows',
     'Int64 type: 1' => 'PHP encodes integers as 32-bit if range allows',
+    'Javascript Code: Embedded nulls' => 'Embedded null in code string is not supported in libbson (CDRIVER-1879)',
     'Javascript Code with Scope: Unicode and embedded null in code string, empty scope' => 'Embedded null in code string is not supported in libbson (CDRIVER-1879)',
     'Multiple types within the same document: All BSON types' => 'PHP encodes integers as 32-bit if range allows',
     'Top-level document validity: Bad $date (number, not string or hash)' => 'Legacy extended JSON $date syntax uses numbers (CDRIVER-2223)',
@@ -150,13 +151,25 @@ function getParamsForValid(array $test, array $case)
     $code .= 'echo bin2hex(fromPHP(toPHP($canonicalBson))), "\n";' . "\n";
     $expect .= $expectedCanonicalBson . "\n";
 
+    $code .= "\n// Canonical BSON -> BSON object -> Canonical BSON\n";
+    $code .= 'echo bin2hex((string) MongoDB\BSON\Document::fromBSON($canonicalBson)), "\n";' . "\n";
+    $expect .= $expectedCanonicalBson . "\n";
+
     $code .= "\n// Canonical BSON -> Canonical extJSON\n";
-    $code .= 'echo json_canonicalize(toCanonicalExtendedJSON($canonicalBson)), "\n";' . "\n";;
+    $code .= 'echo json_canonicalize(toCanonicalExtendedJSON($canonicalBson)), "\n";' . "\n";
+    $expect .= $expectedCanonicalExtJson . "\n";
+
+    $code .= "\n// Canonical BSON -> BSON object -> Canonical extJSON\n";
+    $code .= 'echo json_canonicalize(MongoDB\BSON\Document::fromBSON($canonicalBson)->toCanonicalExtendedJSON()), "\n";' . "\n";
     $expect .= $expectedCanonicalExtJson . "\n";
 
     if (isset($relaxedExtJson)) {
         $code .= "\n// Canonical BSON -> Relaxed extJSON\n";
-        $code .= 'echo json_canonicalize(toRelaxedExtendedJSON($canonicalBson)), "\n";' . "\n";;
+        $code .= 'echo json_canonicalize(toRelaxedExtendedJSON($canonicalBson)), "\n";' . "\n";
+        $expect .= $expectedRelaxedExtJson . "\n";
+
+        $code .= "\n// Canonical BSON -> BSON object -> Relaxed extJSON\n";
+        $code .= 'echo json_canonicalize(MongoDB\BSON\Document::fromBSON($canonicalBson)->toRelaxedExtendedJSON()), "\n";' . "\n";
         $expect .= $expectedRelaxedExtJson . "\n";
     }
 
@@ -164,6 +177,10 @@ function getParamsForValid(array $test, array $case)
         $code .= "\n// Canonical extJSON -> Canonical BSON\n";
         $code .= 'echo bin2hex(fromJSON($canonicalExtJson)), "\n";' . "\n";
         $expect .= $expectedCanonicalBson . "\n";
+
+        $code .= "\n// Canonical extJSON -> BSON object -> Canonical BSON\n";
+        $code .= 'echo bin2hex((string) MongoDB\BSON\Document::fromJSON($canonicalExtJson)), "\n";' . "\n";
+        $expect .= $expectedCanonicalBson . "\n";
     }
 
     if (isset($degenerateBson)) {
@@ -171,13 +188,25 @@ function getParamsForValid(array $test, array $case)
         $code .= 'echo bin2hex(fromPHP(toPHP($degenerateBson))), "\n";' . "\n";
         $expect .= $expectedCanonicalBson . "\n";
 
+        $code .= "\n// Degenerate BSON -> BSON object -> Canonical BSON\n";
+        $code .= 'echo bin2hex(fromPHP(MongoDB\BSON\Document::fromBSON($degenerateBson)->toPHP())), "\n";' . "\n";
+        $expect .= $expectedCanonicalBson . "\n";
+
         $code .= "\n// Degenerate BSON -> Canonical extJSON\n";
-        $code .= 'echo json_canonicalize(toCanonicalExtendedJSON($degenerateBson)), "\n";' . "\n";;
+        $code .= 'echo json_canonicalize(toCanonicalExtendedJSON($degenerateBson)), "\n";' . "\n";
+        $expect .= $expectedCanonicalExtJson . "\n";
+
+        $code .= "\n// Degenerate BSON -> BSON object -> Canonical extJSON\n";
+        $code .= 'echo json_canonicalize(MongoDB\BSON\Document::fromBSON($degenerateBson)->toCanonicalExtendedJSON()), "\n";' . "\n";
         $expect .= $expectedCanonicalExtJson . "\n";
 
         if (isset($relaxedExtJson)) {
             $code .= "\n// Degenerate BSON -> Relaxed extJSON\n";
-            $code .= 'echo json_canonicalize(toRelaxedExtendedJSON($degenerateBson)), "\n";' . "\n";;
+            $code .= 'echo json_canonicalize(toRelaxedExtendedJSON($degenerateBson)), "\n";' . "\n";
+            $expect .= $expectedRelaxedExtJson . "\n";
+
+            $code .= "\n// Degenerate BSON -> BSON object -> Relaxed extJSON\n";
+            $code .= 'echo json_canonicalize(MongoDB\BSON\Document::fromBSON($degenerateBson)->toRelaxedExtendedJSON()), "\n";' . "\n";
             $expect .= $expectedRelaxedExtJson . "\n";
         }
     }
@@ -186,12 +215,20 @@ function getParamsForValid(array $test, array $case)
         $code .= "\n// Degenerate extJSON -> Canonical BSON\n";
         $code .= 'echo bin2hex(fromJSON($degenerateExtJson)), "\n";' . "\n";
         $expect .= $expectedCanonicalBson . "\n";
+
+        $code .= "\n// Degenerate extJSON -> BSON object -> Canonical BSON\n";
+        $code .= 'echo bin2hex((string) MongoDB\BSON\Document::fromJSON($degenerateExtJson)), "\n";' . "\n";
+        $expect .= $expectedCanonicalBson . "\n";
     }
 
     if (isset($relaxedExtJson)) {
         $code .= "\n// Relaxed extJSON -> BSON -> Relaxed extJSON\n";
         $code .= 'echo json_canonicalize(toRelaxedExtendedJSON(fromJSON($relaxedExtJson))), "\n";' . "\n";
         $expect .= $expectedRelaxedExtJson . "\n";
+
+        $code .= "\n// Relaxed extJSON -> BSON object -> Relaxed extJSON\n";
+        $code .= 'echo json_canonicalize(MongoDB\BSON\Document::fromJSON($relaxedExtJson)->toRelaxedExtendedJSON()), "\n";' . "\n";
+        $expect .= $expectedRelaxedExtJson . "\n";
     }
 
     return [
