@@ -23,12 +23,13 @@
 
 #include "php_phongo.h"
 #include "phongo_error.h"
+#include "DBPointer_arginfo.h"
 
 zend_class_entry* php_phongo_dbpointer_ce;
 
 /* Initialize the object and return whether it was successful. An exception will
  * be thrown on error. */
-static bool php_phongo_dbpointer_init(php_phongo_dbpointer_t* intern, const char* ref, size_t ref_len, const char* id, size_t id_len) /* {{{ */
+static bool php_phongo_dbpointer_init(php_phongo_dbpointer_t* intern, const char* ref, size_t ref_len, const char* id, size_t id_len)
 {
 	if (strlen(ref) != (size_t) ref_len) {
 		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Ref cannot contain null bytes");
@@ -47,11 +48,11 @@ static bool php_phongo_dbpointer_init(php_phongo_dbpointer_t* intern, const char
 	strncpy(intern->id, id, sizeof(intern->id) - 1);
 
 	return true;
-} /* }}} */
+}
 
 /* Initialize the object from a HashTable and return whether it was successful.
  * An exception will be thrown on error. */
-static bool php_phongo_dbpointer_init_from_hash(php_phongo_dbpointer_t* intern, HashTable* props) /* {{{ */
+static bool php_phongo_dbpointer_init_from_hash(php_phongo_dbpointer_t* intern, HashTable* props)
 {
 	zval *ref, *id;
 
@@ -63,9 +64,9 @@ static bool php_phongo_dbpointer_init_from_hash(php_phongo_dbpointer_t* intern,
 
 	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "%s initialization requires \"ref\" and \"id\" string fields", ZSTR_VAL(php_phongo_dbpointer_ce->name));
 	return false;
-} /* }}} */
+}
 
-HashTable* php_phongo_dbpointer_get_properties_hash(phongo_compat_object_handler_type* object, bool is_temp) /* {{{ */
+HashTable* php_phongo_dbpointer_get_properties_hash(phongo_compat_object_handler_type* object, bool is_temp)
 {
 	php_phongo_dbpointer_t* intern;
 	HashTable*              props;
@@ -88,46 +89,51 @@ HashTable* php_phongo_dbpointer_get_properties_hash(phongo_compat_object_handler
 	}
 
 	return props;
-} /* }}} */
+}
 
-/* {{{ proto string MongoDB\BSON\DBPointer::__toString()
-   Return the DBPointer's namespace string and ObjectId. */
-static PHP_METHOD(DBPointer, __toString)
+PHONGO_DISABLED_CONSTRUCTOR(MongoDB_BSON_DBPointer)
+
+/* Return the DBPointer's namespace string and ObjectId. */
+static PHP_METHOD(MongoDB_BSON_DBPointer, __toString)
 {
-	zend_error_handling     error_handling;
 	php_phongo_dbpointer_t* intern;
 	char*                   retval;
 	int                     retval_len;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	intern = Z_DBPOINTER_OBJ_P(getThis());
 
 	retval_len = spprintf(&retval, 0, "[%s/%s]", intern->ref, intern->id);
 	RETVAL_STRINGL(retval, retval_len);
 	efree(retval);
-} /* }}} */
+}
 
-/* {{{ proto array MongoDB\BSON\Symbol::jsonSerialize()
-*/
-static PHP_METHOD(DBPointer, jsonSerialize)
+static PHP_METHOD(MongoDB_BSON_DBPointer, __set_state)
+{
+	php_phongo_dbpointer_t* intern;
+	HashTable*              props;
+	zval*                   array;
+
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_ARRAY(array)
+	PHONGO_PARSE_PARAMETERS_END();
+
+	object_init_ex(return_value, php_phongo_dbpointer_ce);
+
+	intern = Z_DBPOINTER_OBJ_P(return_value);
+	props  = Z_ARRVAL_P(array);
+
+	php_phongo_dbpointer_init_from_hash(intern, props);
+}
+
+static PHP_METHOD(MongoDB_BSON_DBPointer, jsonSerialize)
 {
-	zend_error_handling     error_handling;
 	php_phongo_dbpointer_t* intern;
 	zval                    zdb_pointer;
 	zval                    zoid;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	intern = Z_DBPOINTER_OBJ_P(getThis());
 
@@ -139,13 +145,10 @@ static PHP_METHOD(DBPointer, jsonSerialize)
 
 	array_init_size(return_value, 1);
 	ADD_ASSOC_ZVAL(return_value, "$dbPointer", &zdb_pointer);
-} /* }}} */
+}
 
-/* {{{ proto string MongoDB\BSON\DBPointer::serialize()
-*/
-static PHP_METHOD(DBPointer, serialize)
+static PHP_METHOD(MongoDB_BSON_DBPointer, serialize)
 {
-	zend_error_handling     error_handling;
 	php_phongo_dbpointer_t* intern;
 	zval                    retval;
 	php_serialize_data_t    var_hash;
@@ -153,12 +156,7 @@ static PHP_METHOD(DBPointer, serialize)
 
 	intern = Z_DBPOINTER_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	array_init_size(&retval, 2);
 	ADD_ASSOC_STRINGL(&retval, "ref", intern->ref, intern->ref_len);
@@ -173,13 +171,10 @@ static PHP_METHOD(DBPointer, serialize)
 
 	smart_str_free(&buf);
 	zval_ptr_dtor(&retval);
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\BSON\DBPointer::unserialize(string $serialized)
-*/
-static PHP_METHOD(DBPointer, unserialize)
+static PHP_METHOD(MongoDB_BSON_DBPointer, unserialize)
 {
-	zend_error_handling     error_handling;
 	php_phongo_dbpointer_t* intern;
 	char*                   serialized;
 	size_t                  serialized_len;
@@ -188,12 +183,9 @@ static PHP_METHOD(DBPointer, unserialize)
 
 	intern = Z_DBPOINTER_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_STRING(serialized, serialized_len)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	PHP_VAR_UNSERIALIZE_INIT(var_hash);
 	if (!php_var_unserialize(&props, (const unsigned char**) &serialized, (unsigned char*) serialized + serialized_len, &var_hash)) {
@@ -207,20 +199,16 @@ static PHP_METHOD(DBPointer, unserialize)
 
 	php_phongo_dbpointer_init_from_hash(intern, HASH_OF(&props));
 	zval_ptr_dtor(&props);
-} /* }}} */
+}
 
-/* {{{ proto array MongoDB\Driver\DBPointer::__serialize()
-*/
-static PHP_METHOD(DBPointer, __serialize)
+static PHP_METHOD(MongoDB_BSON_DBPointer, __serialize)
 {
 	PHONGO_PARSE_PARAMETERS_NONE();
 
 	RETURN_ARR(php_phongo_dbpointer_get_properties_hash(PHONGO_COMPAT_OBJ_P(getThis()), true));
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\DBPointer::__unserialize(array $data)
-*/
-static PHP_METHOD(DBPointer, __unserialize)
+static PHP_METHOD(MongoDB_BSON_DBPointer, __unserialize)
 {
 	zval* data;
 
@@ -229,45 +217,12 @@ static PHP_METHOD(DBPointer, __unserialize)
 	PHONGO_PARSE_PARAMETERS_END();
 
 	php_phongo_dbpointer_init_from_hash(Z_DBPOINTER_OBJ_P(getThis()), Z_ARRVAL_P(data));
-} /* }}} */
+}
 
-/* {{{ MongoDB\BSON\DBPointer function entries */
-/* clang-format off */
-ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(ai_DBPointer___toString, 0, 0, IS_STRING, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_DBPointer___unserialize, 0, 0, 1)
-	ZEND_ARG_ARRAY_INFO(0, data, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_DBPointer_jsonSerialize, 0, 0, IS_ARRAY, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_DBPointer_unserialize, 0, 0, 1)
-	ZEND_ARG_INFO(0, serialized)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_DBPointer_void, 0, 0, 0)
-ZEND_END_ARG_INFO()
-
-static zend_function_entry php_phongo_dbpointer_me[] = {
-	/* __set_state intentionally missing */
-	PHP_ME(DBPointer, __serialize, ai_DBPointer_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(DBPointer, __toString, ai_DBPointer___toString, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(DBPointer, __unserialize, ai_DBPointer___unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(DBPointer, jsonSerialize, ai_DBPointer_jsonSerialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(DBPointer, serialize, ai_DBPointer_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(DBPointer, unserialize, ai_DBPointer_unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	ZEND_NAMED_ME(__construct, PHP_FN(MongoDB_disabled___construct), ai_DBPointer_void, ZEND_ACC_PRIVATE | ZEND_ACC_FINAL)
-	PHP_FE_END
-};
-/* clang-format on */
-/* }}} */
-
-/* {{{ MongoDB\BSON\DBPointer object handlers */
+/* MongoDB\BSON\DBPointer object handlers */
 static zend_object_handlers php_phongo_handler_dbpointer;
 
-static void php_phongo_dbpointer_free_object(zend_object* object) /* {{{ */
+static void php_phongo_dbpointer_free_object(zend_object* object)
 {
 	php_phongo_dbpointer_t* intern = Z_OBJ_DBPOINTER(object);
 
@@ -281,9 +236,9 @@ static void php_phongo_dbpointer_free_object(zend_object* object) /* {{{ */
 		zend_hash_destroy(intern->properties);
 		FREE_HASHTABLE(intern->properties);
 	}
-} /* }}} */
+}
 
-zend_object* php_phongo_dbpointer_create_object(zend_class_entry* class_type) /* {{{ */
+zend_object* php_phongo_dbpointer_create_object(zend_class_entry* class_type)
 {
 	php_phongo_dbpointer_t* intern = zend_object_alloc(sizeof(php_phongo_dbpointer_t), class_type);
 
@@ -293,9 +248,9 @@ zend_object* php_phongo_dbpointer_create_object(zend_class_entry* class_type) /*
 	intern->std.handlers = &php_phongo_handler_dbpointer;
 
 	return &intern->std;
-} /* }}} */
+}
 
-static zend_object* php_phongo_dbpointer_clone_object(phongo_compat_object_handler_type* object) /* {{{ */
+static zend_object* php_phongo_dbpointer_clone_object(phongo_compat_object_handler_type* object)
 {
 	php_phongo_dbpointer_t* intern;
 	php_phongo_dbpointer_t* new_intern;
@@ -310,9 +265,9 @@ static zend_object* php_phongo_dbpointer_clone_object(phongo_compat_object_handl
 	php_phongo_dbpointer_init(new_intern, intern->ref, intern->ref_len, intern->id, 24);
 
 	return new_object;
-} /* }}} */
+}
 
-static int php_phongo_dbpointer_compare_objects(zval* o1, zval* o2) /* {{{ */
+static int php_phongo_dbpointer_compare_objects(zval* o1, zval* o2)
 {
 	php_phongo_dbpointer_t *intern1, *intern2;
 	int                     retval;
@@ -329,32 +284,23 @@ static int php_phongo_dbpointer_compare_objects(zval* o1, zval* o2) /* {{{ */
 	}
 
 	return strcmp(intern1->id, intern2->id);
-} /* }}} */
+}
 
-static HashTable* php_phongo_dbpointer_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_dbpointer_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp)
 {
 	*is_temp = 1;
 	return php_phongo_dbpointer_get_properties_hash(object, true);
-} /* }}} */
+}
 
-static HashTable* php_phongo_dbpointer_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
+static HashTable* php_phongo_dbpointer_get_properties(phongo_compat_object_handler_type* object)
 {
 	return php_phongo_dbpointer_get_properties_hash(object, false);
-} /* }}} */
-/* }}} */
+}
 
-void php_phongo_dbpointer_init_ce(INIT_FUNC_ARGS) /* {{{ */
+void php_phongo_dbpointer_init_ce(INIT_FUNC_ARGS)
 {
-	zend_class_entry ce;
-
-	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\BSON", "DBPointer", php_phongo_dbpointer_me);
-	php_phongo_dbpointer_ce                = zend_register_internal_class(&ce);
+	php_phongo_dbpointer_ce                = register_class_MongoDB_BSON_DBPointer(php_phongo_json_serializable_ce, php_phongo_type_ce, zend_ce_serializable);
 	php_phongo_dbpointer_ce->create_object = php_phongo_dbpointer_create_object;
-	PHONGO_CE_FINAL(php_phongo_dbpointer_ce);
-
-	zend_class_implements(php_phongo_dbpointer_ce, 1, php_phongo_json_serializable_ce);
-	zend_class_implements(php_phongo_dbpointer_ce, 1, php_phongo_type_ce);
-	zend_class_implements(php_phongo_dbpointer_ce, 1, zend_ce_serializable);
 
 #if PHP_VERSION_ID >= 80000
 	zend_class_implements(php_phongo_dbpointer_ce, 1, zend_ce_stringable);
@@ -367,4 +313,18 @@ void php_phongo_dbpointer_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	php_phongo_handler_dbpointer.get_properties = php_phongo_dbpointer_get_properties;
 	php_phongo_handler_dbpointer.free_obj       = php_phongo_dbpointer_free_object;
 	php_phongo_handler_dbpointer.offset         = XtOffsetOf(php_phongo_dbpointer_t, std);
-} /* }}} */
+}
+
+bool phongo_dbpointer_new(zval* object, const char* ref, size_t ref_len, const bson_oid_t* oid)
+{
+	php_phongo_dbpointer_t* intern;
+
+	object_init_ex(object, php_phongo_dbpointer_ce);
+
+	intern          = Z_DBPOINTER_OBJ_P(object);
+	intern->ref     = estrndup(ref, ref_len);
+	intern->ref_len = ref_len;
+	bson_oid_to_string(oid, intern->id);
+
+	return true;
+}
