@@ -14,17 +14,16 @@
  * limitations under the License.
  */
 
+#include "bson/bson.h"
+
 #include <php.h>
-#include <Zend/zend_interfaces.h>
-#include <ext/standard/php_var.h>
 #include <zend_smart_str.h>
+#include <ext/standard/php_var.h>
+#include <Zend/zend_interfaces.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "phongo_compat.h"
 #include "php_phongo.h"
+#include "phongo_error.h"
+#include "ObjectId_arginfo.h"
 
 #define PHONGO_OID_SIZE sizeof(((php_phongo_objectid_t*) 0)->oid)
 #define PHONGO_OID_LEN (PHONGO_OID_SIZE - 1)
@@ -47,7 +46,7 @@ static bool php_phongo_objectid_init(php_phongo_objectid_t* intern)
 
 /* Initialize the object from a hex string and return whether it was successful.
  * An exception will be thrown on error. */
-static bool php_phongo_objectid_init_from_hex_string(php_phongo_objectid_t* intern, const char* hex, size_t hex_len) /* {{{ */
+static bool php_phongo_objectid_init_from_hex_string(php_phongo_objectid_t* intern, const char* hex, size_t hex_len)
 {
 	if (bson_oid_is_valid(hex, hex_len)) {
 		bson_oid_t oid;
@@ -62,11 +61,11 @@ static bool php_phongo_objectid_init_from_hex_string(php_phongo_objectid_t* inte
 	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Error parsing ObjectId string: %s", hex);
 
 	return false;
-} /* }}} */
+}
 
 /* Initialize the object from a HashTable and return whether it was successful.
  * An exception will be thrown on error. */
-static bool php_phongo_objectid_init_from_hash(php_phongo_objectid_t* intern, HashTable* props) /* {{{ */
+static bool php_phongo_objectid_init_from_hash(php_phongo_objectid_t* intern, HashTable* props)
 {
 	zval* z_oid;
 
@@ -78,69 +77,74 @@ static bool php_phongo_objectid_init_from_hash(php_phongo_objectid_t* intern, Ha
 
 	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "%s initialization requires \"oid\" string field", ZSTR_VAL(php_phongo_objectid_ce->name));
 	return false;
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\BSON\ObjectId::__construct([string $id])
-   Constructs a new BSON ObjectId type, optionally from a hex string. */
-static PHP_METHOD(ObjectId, __construct)
+static HashTable* php_phongo_objectid_get_properties_hash(phongo_compat_object_handler_type* object, bool is_temp)
+{
+	php_phongo_objectid_t* intern;
+	HashTable*             props;
+
+	intern = Z_OBJ_OBJECTID(PHONGO_COMPAT_GET_OBJ(object));
+
+	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_temp, intern, props, 1);
+
+	if (!intern->initialized) {
+		return props;
+	}
+
+	{
+		zval zv;
+
+		ZVAL_STRING(&zv, intern->oid);
+		zend_hash_str_update(props, "oid", sizeof("oid") - 1, &zv);
+	}
+
+	return props;
+}
+
+/* Constructs a new BSON ObjectId type, optionally from a hex string. */
+static PHP_METHOD(MongoDB_BSON_ObjectId, __construct)
 {
-	zend_error_handling    error_handling;
 	php_phongo_objectid_t* intern;
 	char*                  id = NULL;
 	size_t                 id_len;
 
 	intern = Z_OBJECTID_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|s!", &id, &id_len) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(0, 1)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_STRING_OR_NULL(id, id_len)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	if (id) {
 		php_phongo_objectid_init_from_hex_string(intern, id, id_len);
 	} else {
 		php_phongo_objectid_init(intern);
 	}
-} /* }}} */
+}
 
-/* {{{ proto integer MongoDB\BSON\ObjectId::getTimestamp()
-*/
-static PHP_METHOD(ObjectId, getTimestamp)
+static PHP_METHOD(MongoDB_BSON_ObjectId, getTimestamp)
 {
-	zend_error_handling    error_handling;
 	php_phongo_objectid_t* intern;
 	bson_oid_t             tmp_oid;
 
 	intern = Z_OBJECTID_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	bson_oid_init_from_string(&tmp_oid, intern->oid);
 	RETVAL_LONG(bson_oid_get_time_t(&tmp_oid));
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\BSON\ObjectId MongoDB\BSON\ObjectId::__set_state(array $properties)
-*/
-static PHP_METHOD(ObjectId, __set_state)
+static PHP_METHOD(MongoDB_BSON_ObjectId, __set_state)
 {
-	zend_error_handling    error_handling;
 	php_phongo_objectid_t* intern;
 	HashTable*             props;
 	zval*                  array;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &array) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_ARRAY(array)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	object_init_ex(return_value, php_phongo_objectid_ce);
 
@@ -148,52 +152,33 @@ static PHP_METHOD(ObjectId, __set_state)
 	props  = Z_ARRVAL_P(array);
 
 	php_phongo_objectid_init_from_hash(intern, props);
-} /* }}} */
+}
 
-/* {{{ proto string MongoDB\BSON\ObjectId::__toString()
-*/
-static PHP_METHOD(ObjectId, __toString)
+static PHP_METHOD(MongoDB_BSON_ObjectId, __toString)
 {
-	zend_error_handling    error_handling;
 	php_phongo_objectid_t* intern;
 
 	intern = Z_OBJECTID_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	RETURN_STRINGL(intern->oid, PHONGO_OID_LEN);
-} /* }}} */
+}
 
-/* {{{ proto array MongoDB\BSON\ObjectId::jsonSerialize()
-*/
-static PHP_METHOD(ObjectId, jsonSerialize)
+static PHP_METHOD(MongoDB_BSON_ObjectId, jsonSerialize)
 {
-	zend_error_handling    error_handling;
 	php_phongo_objectid_t* intern;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	intern = Z_OBJECTID_OBJ_P(getThis());
 
 	array_init_size(return_value, 1);
 	ADD_ASSOC_STRINGL(return_value, "$oid", intern->oid, PHONGO_OID_LEN);
-} /* }}} */
+}
 
-/* {{{ proto string MongoDB\BSON\ObjectId::serialize()
-*/
-static PHP_METHOD(ObjectId, serialize)
+static PHP_METHOD(MongoDB_BSON_ObjectId, serialize)
 {
-	zend_error_handling    error_handling;
 	php_phongo_objectid_t* intern;
 	zval                   retval;
 	php_serialize_data_t   var_hash;
@@ -201,12 +186,7 @@ static PHP_METHOD(ObjectId, serialize)
 
 	intern = Z_OBJECTID_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	array_init_size(&retval, 1);
 	ADD_ASSOC_STRINGL(&retval, "oid", intern->oid, PHONGO_OID_LEN);
@@ -220,13 +200,10 @@ static PHP_METHOD(ObjectId, serialize)
 
 	smart_str_free(&buf);
 	zval_ptr_dtor(&retval);
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\BSON\ObjectId::unserialize(string $serialized)
-*/
-static PHP_METHOD(ObjectId, unserialize)
+static PHP_METHOD(MongoDB_BSON_ObjectId, unserialize)
 {
-	zend_error_handling    error_handling;
 	php_phongo_objectid_t* intern;
 	char*                  serialized;
 	size_t                 serialized_len;
@@ -235,12 +212,9 @@ static PHP_METHOD(ObjectId, unserialize)
 
 	intern = Z_OBJECTID_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_STRING(serialized, serialized_len)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	PHP_VAR_UNSERIALIZE_INIT(var_hash);
 	if (!php_var_unserialize(&props, (const unsigned char**) &serialized, (unsigned char*) serialized + serialized_len, &var_hash)) {
@@ -254,42 +228,30 @@ static PHP_METHOD(ObjectId, unserialize)
 
 	php_phongo_objectid_init_from_hash(intern, HASH_OF(&props));
 	zval_ptr_dtor(&props);
-} /* }}} */
+}
 
-/* {{{ MongoDB\BSON\ObjectId function entries */
-ZEND_BEGIN_ARG_INFO_EX(ai_ObjectId___construct, 0, 0, 0)
-	ZEND_ARG_INFO(0, id)
-ZEND_END_ARG_INFO()
+static PHP_METHOD(MongoDB_BSON_ObjectId, __serialize)
+{
+	PHONGO_PARSE_PARAMETERS_NONE();
 
-ZEND_BEGIN_ARG_INFO_EX(ai_ObjectId___set_state, 0, 0, 1)
-	ZEND_ARG_ARRAY_INFO(0, properties, 0)
-ZEND_END_ARG_INFO()
+	RETURN_ARR(php_phongo_objectid_get_properties_hash(PHONGO_COMPAT_OBJ_P(getThis()), true));
+}
 
-ZEND_BEGIN_ARG_INFO_EX(ai_ObjectId_unserialize, 0, 0, 1)
-	ZEND_ARG_INFO(0, serialized)
-ZEND_END_ARG_INFO()
+static PHP_METHOD(MongoDB_BSON_ObjectId, __unserialize)
+{
+	zval* data;
 
-ZEND_BEGIN_ARG_INFO_EX(ai_ObjectId_void, 0, 0, 0)
-ZEND_END_ARG_INFO()
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_ARRAY(data)
+	PHONGO_PARSE_PARAMETERS_END();
 
-static zend_function_entry php_phongo_objectid_me[] = {
-	/* clang-format off */
-	PHP_ME(ObjectId, __construct, ai_ObjectId___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(ObjectId, getTimestamp, ai_ObjectId_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(ObjectId, __set_state, ai_ObjectId___set_state, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-	PHP_ME(ObjectId, __toString, ai_ObjectId_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(ObjectId, jsonSerialize, ai_ObjectId_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(ObjectId, serialize, ai_ObjectId_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(ObjectId, unserialize, ai_ObjectId_unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_FE_END
-	/* clang-format on */
-};
-/* }}} */
+	php_phongo_objectid_init_from_hash(Z_OBJECTID_OBJ_P(getThis()), Z_ARRVAL_P(data));
+}
 
-/* {{{ MongoDB\BSON\ObjectId object handlers */
+/* MongoDB\BSON\ObjectId object handlers */
 static zend_object_handlers php_phongo_handler_objectid;
 
-static void php_phongo_objectid_free_object(zend_object* object) /* {{{ */
+static void php_phongo_objectid_free_object(zend_object* object)
 {
 	php_phongo_objectid_t* intern = Z_OBJ_OBJECTID(object);
 
@@ -299,13 +261,11 @@ static void php_phongo_objectid_free_object(zend_object* object) /* {{{ */
 		zend_hash_destroy(intern->properties);
 		FREE_HASHTABLE(intern->properties);
 	}
-} /* }}} */
+}
 
-static zend_object* php_phongo_objectid_create_object(zend_class_entry* class_type) /* {{{ */
+static zend_object* php_phongo_objectid_create_object(zend_class_entry* class_type)
 {
-	php_phongo_objectid_t* intern = NULL;
-
-	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_objectid_t, class_type);
+	php_phongo_objectid_t* intern = zend_object_alloc(sizeof(php_phongo_objectid_t), class_type);
 
 	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
@@ -313,9 +273,9 @@ static zend_object* php_phongo_objectid_create_object(zend_class_entry* class_ty
 	intern->std.handlers = &php_phongo_handler_objectid;
 
 	return &intern->std;
-} /* }}} */
+}
 
-static zend_object* php_phongo_objectid_clone_object(phongo_compat_object_handler_type* object) /* {{{ */
+static zend_object* php_phongo_objectid_clone_object(phongo_compat_object_handler_type* object)
 {
 	php_phongo_objectid_t* intern;
 	php_phongo_objectid_t* new_intern;
@@ -332,9 +292,9 @@ static zend_object* php_phongo_objectid_clone_object(phongo_compat_object_handle
 	new_intern->initialized = true;
 
 	return new_object;
-} /* }}} */
+}
 
-static int php_phongo_objectid_compare_objects(zval* o1, zval* o2) /* {{{ */
+static int php_phongo_objectid_compare_objects(zval* o1, zval* o2)
 {
 	php_phongo_objectid_t* intern1;
 	php_phongo_objectid_t* intern2;
@@ -345,56 +305,27 @@ static int php_phongo_objectid_compare_objects(zval* o1, zval* o2) /* {{{ */
 	intern2 = Z_OBJECTID_OBJ_P(o2);
 
 	return strcmp(intern1->oid, intern2->oid);
-} /* }}} */
+}
 
-static HashTable* php_phongo_objectid_get_properties_hash(phongo_compat_object_handler_type* object, bool is_debug) /* {{{ */
-{
-	php_phongo_objectid_t* intern;
-	HashTable*             props;
-
-	intern = Z_OBJ_OBJECTID(PHONGO_COMPAT_GET_OBJ(object));
-
-	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 1);
-
-	if (!intern->initialized) {
-		return props;
-	}
-
-	{
-		zval zv;
-
-		ZVAL_STRING(&zv, intern->oid);
-		zend_hash_str_update(props, "oid", sizeof("oid") - 1, &zv);
-	}
-
-	return props;
-} /* }}} */
-
-static HashTable* php_phongo_objectid_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_objectid_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp)
 {
 	*is_temp = 1;
 	return php_phongo_objectid_get_properties_hash(object, true);
-} /* }}} */
+}
 
-static HashTable* php_phongo_objectid_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
+static HashTable* php_phongo_objectid_get_properties(phongo_compat_object_handler_type* object)
 {
 	return php_phongo_objectid_get_properties_hash(object, false);
-} /* }}} */
-/* }}} */
+}
 
-void php_phongo_objectid_init_ce(INIT_FUNC_ARGS) /* {{{ */
+void php_phongo_objectid_init_ce(INIT_FUNC_ARGS)
 {
-	zend_class_entry ce;
-
-	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\BSON", "ObjectId", php_phongo_objectid_me);
-	php_phongo_objectid_ce                = zend_register_internal_class(&ce);
+	php_phongo_objectid_ce                = register_class_MongoDB_BSON_ObjectId(php_phongo_objectid_interface_ce, php_phongo_json_serializable_ce, php_phongo_type_ce, zend_ce_serializable);
 	php_phongo_objectid_ce->create_object = php_phongo_objectid_create_object;
-	PHONGO_CE_FINAL(php_phongo_objectid_ce);
 
-	zend_class_implements(php_phongo_objectid_ce, 1, php_phongo_objectid_interface_ce);
-	zend_class_implements(php_phongo_objectid_ce, 1, php_phongo_json_serializable_ce);
-	zend_class_implements(php_phongo_objectid_ce, 1, php_phongo_type_ce);
-	zend_class_implements(php_phongo_objectid_ce, 1, zend_ce_serializable);
+#if PHP_VERSION_ID >= 80000
+	zend_class_implements(php_phongo_objectid_ce, 1, zend_ce_stringable);
+#endif
 
 	memcpy(&php_phongo_handler_objectid, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	PHONGO_COMPAT_SET_COMPARE_OBJECTS_HANDLER(objectid);
@@ -403,13 +334,17 @@ void php_phongo_objectid_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	php_phongo_handler_objectid.get_properties = php_phongo_objectid_get_properties;
 	php_phongo_handler_objectid.free_obj       = php_phongo_objectid_free_object;
 	php_phongo_handler_objectid.offset         = XtOffsetOf(php_phongo_objectid_t, std);
-} /* }}} */
+}
 
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
+bool phongo_objectid_new(zval* return_value, const bson_oid_t* oid)
+{
+	php_phongo_objectid_t* intern;
+
+	object_init_ex(return_value, php_phongo_objectid_ce);
+
+	intern = Z_OBJECTID_OBJ_P(return_value);
+	bson_oid_to_string(oid, intern->oid);
+	intern->initialized = true;
+
+	return true;
+}
