@@ -22,12 +22,13 @@
 #include "php_phongo.h"
 #include "phongo_error.h"
 #include "phongo_util.h"
+#include "Timestamp_arginfo.h"
 
 zend_class_entry* php_phongo_timestamp_ce;
 
 /* Initialize the object and return whether it was successful. An exception will
  * be thrown on error. */
-static bool php_phongo_timestamp_init(php_phongo_timestamp_t* intern, int64_t increment, int64_t timestamp) /* {{{ */
+static bool php_phongo_timestamp_init(php_phongo_timestamp_t* intern, int64_t increment, int64_t timestamp)
 {
 	if (increment < 0 || increment > UINT32_MAX) {
 		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected increment to be an unsigned 32-bit integer, %" PRId64 " given", increment);
@@ -44,11 +45,11 @@ static bool php_phongo_timestamp_init(php_phongo_timestamp_t* intern, int64_t in
 	intern->initialized = true;
 
 	return true;
-} /* }}} */
+}
 
 /* Initialize the object from numeric strings and return whether it was
  * successful. An exception will be thrown on error. */
-static bool php_phongo_timestamp_init_from_string(php_phongo_timestamp_t* intern, const char* s_increment, size_t s_increment_len, const char* s_timestamp, size_t s_timestamp_len) /* {{{ */
+static bool php_phongo_timestamp_init_from_string(php_phongo_timestamp_t* intern, const char* s_increment, size_t s_increment_len, const char* s_timestamp, size_t s_timestamp_len)
 {
 	int64_t increment, timestamp;
 
@@ -63,11 +64,11 @@ static bool php_phongo_timestamp_init_from_string(php_phongo_timestamp_t* intern
 	}
 
 	return php_phongo_timestamp_init(intern, increment, timestamp);
-} /* }}} */
+}
 
 /* Initialize the object from a HashTable and return whether it was successful.
  * An exception will be thrown on error. */
-static bool php_phongo_timestamp_init_from_hash(php_phongo_timestamp_t* intern, HashTable* props) /* {{{ */
+static bool php_phongo_timestamp_init_from_hash(php_phongo_timestamp_t* intern, HashTable* props)
 {
 	zval *increment, *timestamp;
 
@@ -84,9 +85,9 @@ static bool php_phongo_timestamp_init_from_hash(php_phongo_timestamp_t* intern,
 
 	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "%s initialization requires \"increment\" and \"timestamp\" integer or numeric string fields", ZSTR_VAL(php_phongo_timestamp_ce->name));
 	return false;
-} /* }}} */
+}
 
-static HashTable* php_phongo_timestamp_get_properties_hash(phongo_compat_object_handler_type* object, bool is_temp) /* {{{ */
+static HashTable* php_phongo_timestamp_get_properties_hash(phongo_compat_object_handler_type* object, bool is_temp)
 {
 	php_phongo_timestamp_t* intern;
 	HashTable*              props;
@@ -117,25 +118,21 @@ static HashTable* php_phongo_timestamp_get_properties_hash(phongo_compat_object_
 	}
 
 	return props;
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\BSON\Timestamp::__construct(int|string $increment, int|string $timestamp)
-   Construct a new BSON timestamp type, which consists of a 4-byte increment and
+/* Construct a new BSON timestamp type, which consists of a 4-byte increment and
    4-byte timestamp. */
-static PHP_METHOD(Timestamp, __construct)
+static PHP_METHOD(MongoDB_BSON_Timestamp, __construct)
 {
-	zend_error_handling     error_handling;
 	php_phongo_timestamp_t* intern;
 	zval *                  increment = NULL, *timestamp = NULL;
 
 	intern = Z_TIMESTAMP_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "zz", &increment, &timestamp) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(2, 2)
+	Z_PARAM_ZVAL(increment)
+	Z_PARAM_ZVAL(timestamp)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	if (Z_TYPE_P(increment) == IS_LONG && Z_TYPE_P(timestamp) == IS_LONG) {
 		php_phongo_timestamp_init(intern, Z_LVAL_P(increment), Z_LVAL_P(timestamp));
@@ -161,61 +158,39 @@ static PHP_METHOD(Timestamp, __construct)
 	}
 
 	php_phongo_timestamp_init_from_string(intern, Z_STRVAL_P(increment), Z_STRLEN_P(increment), Z_STRVAL_P(timestamp), Z_STRLEN_P(timestamp));
-} /* }}} */
+}
 
-/* {{{ proto integer MongoDB\BSON\Timestamp::getIncrement()
-*/
-static PHP_METHOD(Timestamp, getIncrement)
+static PHP_METHOD(MongoDB_BSON_Timestamp, getIncrement)
 {
-	zend_error_handling     error_handling;
 	php_phongo_timestamp_t* intern;
 
 	intern = Z_TIMESTAMP_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	RETVAL_LONG(intern->increment);
-} /* }}} */
+}
 
-/* {{{ proto integer MongoDB\BSON\Timestamp::getTimestamp()
-*/
-static PHP_METHOD(Timestamp, getTimestamp)
+static PHP_METHOD(MongoDB_BSON_Timestamp, getTimestamp)
 {
-	zend_error_handling     error_handling;
 	php_phongo_timestamp_t* intern;
 
 	intern = Z_TIMESTAMP_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	RETVAL_LONG(intern->timestamp);
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\BSON\Timestamp MongoDB\BSON\Timestamp::__set_state(array $properties)
-*/
-static PHP_METHOD(Timestamp, __set_state)
+static PHP_METHOD(MongoDB_BSON_Timestamp, __set_state)
 {
-	zend_error_handling     error_handling;
 	php_phongo_timestamp_t* intern;
 	HashTable*              props;
 	zval*                   array;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &array) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_ARRAY(array)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	object_init_ex(return_value, php_phongo_timestamp_ce);
 
@@ -223,44 +198,29 @@ static PHP_METHOD(Timestamp, __set_state)
 	props  = Z_ARRVAL_P(array);
 
 	php_phongo_timestamp_init_from_hash(intern, props);
-} /* }}} */
+}
 
-/* {{{ proto string MongoDB\BSON\Timestamp::__toString()
-   Returns a string in the form: [increment:timestamp] */
-static PHP_METHOD(Timestamp, __toString)
+/* Returns a string in the form: [increment:timestamp] */
+static PHP_METHOD(MongoDB_BSON_Timestamp, __toString)
 {
-	zend_error_handling     error_handling;
 	php_phongo_timestamp_t* intern;
 	char*                   retval;
 	int                     retval_len;
 
 	intern = Z_TIMESTAMP_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	retval_len = spprintf(&retval, 0, "[%" PRIu32 ":%" PRIu32 "]", intern->increment, intern->timestamp);
 	RETVAL_STRINGL(retval, retval_len);
 	efree(retval);
-} /* }}} */
+}
 
-/* {{{ proto array MongoDB\BSON\Timestamp::jsonSerialize()
-*/
-static PHP_METHOD(Timestamp, jsonSerialize)
+static PHP_METHOD(MongoDB_BSON_Timestamp, jsonSerialize)
 {
-	zend_error_handling     error_handling;
 	php_phongo_timestamp_t* intern;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	intern = Z_TIMESTAMP_OBJ_P(getThis());
 
@@ -274,13 +234,10 @@ static PHP_METHOD(Timestamp, jsonSerialize)
 		ADD_ASSOC_LONG_EX(&ts, "i", intern->increment);
 		ADD_ASSOC_ZVAL_EX(return_value, "$timestamp", &ts);
 	}
-} /* }}} */
+}
 
-/* {{{ proto string MongoDB\BSON\Timestamp::serialize()
-*/
-static PHP_METHOD(Timestamp, serialize)
+static PHP_METHOD(MongoDB_BSON_Timestamp, serialize)
 {
-	zend_error_handling     error_handling;
 	php_phongo_timestamp_t* intern;
 	zval                    retval;
 	php_serialize_data_t    var_hash;
@@ -292,12 +249,7 @@ static PHP_METHOD(Timestamp, serialize)
 
 	intern = Z_TIMESTAMP_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	s_increment_len = snprintf(s_increment, sizeof(s_increment), "%" PRIu32, intern->increment);
 	s_timestamp_len = snprintf(s_timestamp, sizeof(s_timestamp), "%" PRIu32, intern->timestamp);
@@ -315,13 +267,10 @@ static PHP_METHOD(Timestamp, serialize)
 
 	smart_str_free(&buf);
 	zval_ptr_dtor(&retval);
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\BSON\Timestamp::unserialize(string $serialized)
-*/
-static PHP_METHOD(Timestamp, unserialize)
+static PHP_METHOD(MongoDB_BSON_Timestamp, unserialize)
 {
-	zend_error_handling     error_handling;
 	php_phongo_timestamp_t* intern;
 	char*                   serialized;
 	size_t                  serialized_len;
@@ -330,12 +279,9 @@ static PHP_METHOD(Timestamp, unserialize)
 
 	intern = Z_TIMESTAMP_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_STRING(serialized, serialized_len)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	PHP_VAR_UNSERIALIZE_INIT(var_hash);
 	if (!php_var_unserialize(&props, (const unsigned char**) &serialized, (unsigned char*) serialized + serialized_len, &var_hash)) {
@@ -349,20 +295,16 @@ static PHP_METHOD(Timestamp, unserialize)
 
 	php_phongo_timestamp_init_from_hash(intern, HASH_OF(&props));
 	zval_ptr_dtor(&props);
-} /* }}} */
+}
 
-/* {{{ proto array MongoDB\Driver\Timestamp::__serialize()
-*/
-static PHP_METHOD(Timestamp, __serialize)
+static PHP_METHOD(MongoDB_BSON_Timestamp, __serialize)
 {
 	PHONGO_PARSE_PARAMETERS_NONE();
 
 	RETURN_ARR(php_phongo_timestamp_get_properties_hash(PHONGO_COMPAT_OBJ_P(getThis()), true));
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Timestamp::__unserialize(array $data)
-*/
-static PHP_METHOD(Timestamp, __unserialize)
+static PHP_METHOD(MongoDB_BSON_Timestamp, __unserialize)
 {
 	zval* data;
 
@@ -371,56 +313,12 @@ static PHP_METHOD(Timestamp, __unserialize)
 	PHONGO_PARSE_PARAMETERS_END();
 
 	php_phongo_timestamp_init_from_hash(Z_TIMESTAMP_OBJ_P(getThis()), Z_ARRVAL_P(data));
-} /* }}} */
+}
 
-/* {{{ MongoDB\BSON\Timestamp function entries */
-/* clang-format off */
-ZEND_BEGIN_ARG_INFO_EX(ai_Timestamp___construct, 0, 0, 2)
-	ZEND_ARG_INFO(0, increment)
-	ZEND_ARG_INFO(0, timestamp)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Timestamp___set_state, 0, 0, 1)
-	ZEND_ARG_ARRAY_INFO(0, properties, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(ai_Timestamp___toString, 0, 0, IS_STRING, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Timestamp___unserialize, 0, 0, 1)
-	ZEND_ARG_ARRAY_INFO(0, data, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_Timestamp_jsonSerialize, 0, 0, IS_ARRAY, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Timestamp_unserialize, 0, 0, 1)
-	ZEND_ARG_INFO(0, serialized)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Timestamp_void, 0, 0, 0)
-ZEND_END_ARG_INFO()
-
-static zend_function_entry php_phongo_timestamp_me[] = {
-	PHP_ME(Timestamp, __construct, ai_Timestamp___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Timestamp, __serialize, ai_Timestamp_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Timestamp, __set_state, ai_Timestamp___set_state, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-	PHP_ME(Timestamp, __toString, ai_Timestamp___toString, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Timestamp, __unserialize, ai_Timestamp___unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Timestamp, jsonSerialize, ai_Timestamp_jsonSerialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Timestamp, serialize, ai_Timestamp_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Timestamp, unserialize, ai_Timestamp_unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Timestamp, getIncrement, ai_Timestamp_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Timestamp, getTimestamp, ai_Timestamp_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_FE_END
-};
-/* clang-format on */
-/* }}} */
-
-/* {{{ MongoDB\BSON\Timestamp object handlers */
+/* MongoDB\BSON\Timestamp object handlers */
 static zend_object_handlers php_phongo_handler_timestamp;
 
-static void php_phongo_timestamp_free_object(zend_object* object) /* {{{ */
+static void php_phongo_timestamp_free_object(zend_object* object)
 {
 	php_phongo_timestamp_t* intern = Z_OBJ_TIMESTAMP(object);
 
@@ -430,9 +328,9 @@ static void php_phongo_timestamp_free_object(zend_object* object) /* {{{ */
 		zend_hash_destroy(intern->properties);
 		FREE_HASHTABLE(intern->properties);
 	}
-} /* }}} */
+}
 
-static zend_object* php_phongo_timestamp_create_object(zend_class_entry* class_type) /* {{{ */
+static zend_object* php_phongo_timestamp_create_object(zend_class_entry* class_type)
 {
 	php_phongo_timestamp_t* intern = zend_object_alloc(sizeof(php_phongo_timestamp_t), class_type);
 
@@ -442,9 +340,9 @@ static zend_object* php_phongo_timestamp_create_object(zend_class_entry* class_t
 	intern->std.handlers = &php_phongo_handler_timestamp;
 
 	return &intern->std;
-} /* }}} */
+}
 
-static zend_object* php_phongo_timestamp_clone_object(phongo_compat_object_handler_type* object) /* {{{ */
+static zend_object* php_phongo_timestamp_clone_object(phongo_compat_object_handler_type* object)
 {
 	php_phongo_timestamp_t* intern;
 	php_phongo_timestamp_t* new_intern;
@@ -459,9 +357,9 @@ static zend_object* php_phongo_timestamp_clone_object(phongo_compat_object_handl
 	php_phongo_timestamp_init(new_intern, intern->increment, intern->timestamp);
 
 	return new_object;
-} /* }}} */
+}
 
-static int php_phongo_timestamp_compare_objects(zval* o1, zval* o2) /* {{{ */
+static int php_phongo_timestamp_compare_objects(zval* o1, zval* o2)
 {
 	php_phongo_timestamp_t *intern1, *intern2;
 
@@ -480,33 +378,23 @@ static int php_phongo_timestamp_compare_objects(zval* o1, zval* o2) /* {{{ */
 	}
 
 	return 0;
-} /* }}} */
+}
 
-static HashTable* php_phongo_timestamp_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_timestamp_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp)
 {
 	*is_temp = 1;
 	return php_phongo_timestamp_get_properties_hash(object, true);
-} /* }}} */
+}
 
-static HashTable* php_phongo_timestamp_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
+static HashTable* php_phongo_timestamp_get_properties(phongo_compat_object_handler_type* object)
 {
 	return php_phongo_timestamp_get_properties_hash(object, false);
-} /* }}} */
-/* }}} */
+}
 
-void php_phongo_timestamp_init_ce(INIT_FUNC_ARGS) /* {{{ */
+void php_phongo_timestamp_init_ce(INIT_FUNC_ARGS)
 {
-	zend_class_entry ce;
-
-	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\BSON", "Timestamp", php_phongo_timestamp_me);
-	php_phongo_timestamp_ce                = zend_register_internal_class(&ce);
+	php_phongo_timestamp_ce                = register_class_MongoDB_BSON_Timestamp(php_phongo_timestamp_interface_ce, php_phongo_json_serializable_ce, php_phongo_type_ce, zend_ce_serializable);
 	php_phongo_timestamp_ce->create_object = php_phongo_timestamp_create_object;
-	PHONGO_CE_FINAL(php_phongo_timestamp_ce);
-
-	zend_class_implements(php_phongo_timestamp_ce, 1, php_phongo_timestamp_interface_ce);
-	zend_class_implements(php_phongo_timestamp_ce, 1, php_phongo_json_serializable_ce);
-	zend_class_implements(php_phongo_timestamp_ce, 1, php_phongo_type_ce);
-	zend_class_implements(php_phongo_timestamp_ce, 1, zend_ce_serializable);
 
 #if PHP_VERSION_ID >= 80000
 	zend_class_implements(php_phongo_timestamp_ce, 1, zend_ce_stringable);
@@ -519,4 +407,18 @@ void php_phongo_timestamp_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	php_phongo_handler_timestamp.get_properties = php_phongo_timestamp_get_properties;
 	php_phongo_handler_timestamp.free_obj       = php_phongo_timestamp_free_object;
 	php_phongo_handler_timestamp.offset         = XtOffsetOf(php_phongo_timestamp_t, std);
-} /* }}} */
+}
+
+bool phongo_timestamp_new(zval* object, uint32_t increment, uint32_t timestamp)
+{
+	php_phongo_timestamp_t* intern;
+
+	object_init_ex(object, php_phongo_timestamp_ce);
+
+	intern              = Z_TIMESTAMP_OBJ_P(object);
+	intern->increment   = increment;
+	intern->timestamp   = timestamp;
+	intern->initialized = true;
+
+	return true;
+}
