@@ -25,21 +25,22 @@
 #include "php_phongo.h"
 #include "phongo_error.h"
 #include "phongo_util.h"
+#include "UTCDateTime_arginfo.h"
 
 zend_class_entry* php_phongo_utcdatetime_ce;
 
 /* Initialize the object and return whether it was successful. */
-static bool php_phongo_utcdatetime_init(php_phongo_utcdatetime_t* intern, int64_t milliseconds) /* {{{ */
+static bool php_phongo_utcdatetime_init(php_phongo_utcdatetime_t* intern, int64_t milliseconds)
 {
 	intern->milliseconds = milliseconds;
 	intern->initialized  = true;
 
 	return true;
-} /* }}} */
+}
 
 /* Initialize the object from a numeric string and return whether it was
  * successful. An exception will be thrown on error. */
-static bool php_phongo_utcdatetime_init_from_string(php_phongo_utcdatetime_t* intern, const char* s_milliseconds, size_t s_milliseconds_len) /* {{{ */
+static bool php_phongo_utcdatetime_init_from_string(php_phongo_utcdatetime_t* intern, const char* s_milliseconds, size_t s_milliseconds_len)
 {
 	int64_t milliseconds;
 
@@ -49,11 +50,11 @@ static bool php_phongo_utcdatetime_init_from_string(php_phongo_utcdatetime_t* in
 	}
 
 	return php_phongo_utcdatetime_init(intern, milliseconds);
-} /* }}} */
+}
 
 /* Initialize the object from a HashTable and return whether it was successful.
  * An exception will be thrown on error. */
-static bool php_phongo_utcdatetime_init_from_hash(php_phongo_utcdatetime_t* intern, HashTable* props) /* {{{ */
+static bool php_phongo_utcdatetime_init_from_hash(php_phongo_utcdatetime_t* intern, HashTable* props)
 {
 	zval* milliseconds;
 
@@ -67,11 +68,11 @@ static bool php_phongo_utcdatetime_init_from_hash(php_phongo_utcdatetime_t* inte
 
 	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "%s initialization requires \"milliseconds\" integer or numeric string field", ZSTR_VAL(php_phongo_utcdatetime_ce->name));
 	return false;
-} /* }}} */
+}
 
 /* Initialize the object from the current time and return whether it was
  * successful. */
-static bool php_phongo_utcdatetime_init_from_current_time(php_phongo_utcdatetime_t* intern) /* {{{ */
+static bool php_phongo_utcdatetime_init_from_current_time(php_phongo_utcdatetime_t* intern)
 {
 	int64_t        sec, usec;
 	struct timeval cur_time;
@@ -84,11 +85,11 @@ static bool php_phongo_utcdatetime_init_from_current_time(php_phongo_utcdatetime
 	intern->initialized  = true;
 
 	return true;
-} /* }}} */
+}
 
 /* Initialize the object from a DateTime object and return whether it was
  * successful. */
-static bool php_phongo_utcdatetime_init_from_date(php_phongo_utcdatetime_t* intern, php_date_obj* datetime_obj) /* {{{ */
+static bool php_phongo_utcdatetime_init_from_date(php_phongo_utcdatetime_t* intern, php_date_obj* datetime_obj)
 {
 	int64_t sec, usec;
 
@@ -100,9 +101,9 @@ static bool php_phongo_utcdatetime_init_from_date(php_phongo_utcdatetime_t* inte
 	intern->initialized  = true;
 
 	return true;
-} /* }}} */
+}
 
-static HashTable* php_phongo_utcdatetime_get_properties_hash(phongo_compat_object_handler_type* object, bool is_temp) /* {{{ */
+static HashTable* php_phongo_utcdatetime_get_properties_hash(phongo_compat_object_handler_type* object, bool is_temp)
 {
 	php_phongo_utcdatetime_t* intern;
 	HashTable*                props;
@@ -123,26 +124,22 @@ static HashTable* php_phongo_utcdatetime_get_properties_hash(phongo_compat_objec
 	}
 
 	return props;
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\BSON\UTCDateTime::__construct([int|float|string|DateTimeInterface $milliseconds = null])
-   Construct a new BSON UTCDateTime type from either the current time,
+/* Construct a new BSON UTCDateTime type from either the current time,
    milliseconds since the epoch, or a DateTimeInterface object. Defaults to the
    current time. */
-static PHP_METHOD(UTCDateTime, __construct)
+static PHP_METHOD(MongoDB_BSON_UTCDateTime, __construct)
 {
-	zend_error_handling       error_handling;
 	php_phongo_utcdatetime_t* intern;
 	zval*                     milliseconds = NULL;
 
 	intern = Z_UTCDATETIME_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|z!", &milliseconds) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(0, 1)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_ZVAL_OR_NULL(milliseconds)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	if (milliseconds == NULL) {
 		php_phongo_utcdatetime_init_from_current_time(intern);
@@ -150,9 +147,7 @@ static PHP_METHOD(UTCDateTime, __construct)
 	}
 
 	if (Z_TYPE_P(milliseconds) == IS_OBJECT) {
-		if (instanceof_function(Z_OBJCE_P(milliseconds), php_date_get_date_ce()) ||
-			(php_phongo_date_immutable_ce && instanceof_function(Z_OBJCE_P(milliseconds), php_phongo_date_immutable_ce))) {
-
+		if (instanceof_function(Z_OBJCE_P(milliseconds), php_date_get_interface_ce())) {
 			php_phongo_utcdatetime_init_from_date(intern, Z_PHPDATE_P(milliseconds));
 		} else {
 			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected instance of DateTimeInterface, %s given", ZSTR_VAL(Z_OBJCE_P(milliseconds)->name));
@@ -181,23 +176,17 @@ static PHP_METHOD(UTCDateTime, __construct)
 	}
 
 	php_phongo_utcdatetime_init_from_string(intern, Z_STRVAL_P(milliseconds), Z_STRLEN_P(milliseconds));
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\BSON\UTCDateTime MongoDB\BSON\UTCDateTime::__set_state(array $properties)
-*/
-static PHP_METHOD(UTCDateTime, __set_state)
+static PHP_METHOD(MongoDB_BSON_UTCDateTime, __set_state)
 {
-	zend_error_handling       error_handling;
 	php_phongo_utcdatetime_t* intern;
 	HashTable*                props;
 	zval*                     array;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &array) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_ARRAY(array)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	object_init_ex(return_value, php_phongo_utcdatetime_ce);
 
@@ -205,32 +194,23 @@ static PHP_METHOD(UTCDateTime, __set_state)
 	props  = Z_ARRVAL_P(array);
 
 	php_phongo_utcdatetime_init_from_hash(intern, props);
-} /* }}} */
+}
 
-/* {{{ proto string MongoDB\BSON\UTCDateTime::__toString()
-   Returns the UTCDateTime's milliseconds as a string */
-static PHP_METHOD(UTCDateTime, __toString)
+/* Returns the UTCDateTime's milliseconds as a string */
+static PHP_METHOD(MongoDB_BSON_UTCDateTime, __toString)
 {
-	zend_error_handling       error_handling;
 	php_phongo_utcdatetime_t* intern;
 
 	intern = Z_UTCDATETIME_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	ZVAL_INT64_STRING(return_value, intern->milliseconds);
-} /* }}} */
+}
 
-/* {{{ proto DateTime MongoDB\BSON\UTCDateTime::toDateTime()
-   Returns a DateTime object representing this UTCDateTime */
-static PHP_METHOD(UTCDateTime, toDateTime)
+/* Returns a DateTime object representing this UTCDateTime */
+static PHP_METHOD(MongoDB_BSON_UTCDateTime, toDateTime)
 {
-	zend_error_handling       error_handling;
 	php_phongo_utcdatetime_t* intern;
 	php_date_obj*             datetime_obj;
 	char*                     sec;
@@ -238,12 +218,7 @@ static PHP_METHOD(UTCDateTime, toDateTime)
 
 	intern = Z_UTCDATETIME_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	object_init_ex(return_value, php_date_get_date_ce());
 	datetime_obj = Z_PHPDATE_P(return_value);
@@ -254,21 +229,12 @@ static PHP_METHOD(UTCDateTime, toDateTime)
 
 	datetime_obj->time->us = (intern->milliseconds % 1000) * 1000;
 }
-/* }}} */
 
-/* {{{ proto array MongoDB\BSON\UTCDateTime::jsonSerialize()
-*/
-static PHP_METHOD(UTCDateTime, jsonSerialize)
+static PHP_METHOD(MongoDB_BSON_UTCDateTime, jsonSerialize)
 {
-	zend_error_handling       error_handling;
 	php_phongo_utcdatetime_t* intern;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	intern = Z_UTCDATETIME_OBJ_P(getThis());
 
@@ -281,13 +247,10 @@ static PHP_METHOD(UTCDateTime, jsonSerialize)
 		ADD_ASSOC_INT64_AS_STRING(&udt, "$numberLong", intern->milliseconds);
 		ADD_ASSOC_ZVAL_EX(return_value, "$date", &udt);
 	}
-} /* }}} */
+}
 
-/* {{{ proto string MongoDB\BSON\UTCDateTime::serialize()
-*/
-static PHP_METHOD(UTCDateTime, serialize)
+static PHP_METHOD(MongoDB_BSON_UTCDateTime, serialize)
 {
-	zend_error_handling       error_handling;
 	php_phongo_utcdatetime_t* intern;
 	zval                      retval;
 	php_serialize_data_t      var_hash;
@@ -295,12 +258,7 @@ static PHP_METHOD(UTCDateTime, serialize)
 
 	intern = Z_UTCDATETIME_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	array_init_size(&retval, 1);
 	ADD_ASSOC_INT64_AS_STRING(&retval, "milliseconds", intern->milliseconds);
@@ -314,13 +272,10 @@ static PHP_METHOD(UTCDateTime, serialize)
 
 	smart_str_free(&buf);
 	zval_ptr_dtor(&retval);
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\BSON\UTCDateTime::unserialize(string $serialized)
-*/
-static PHP_METHOD(UTCDateTime, unserialize)
+static PHP_METHOD(MongoDB_BSON_UTCDateTime, unserialize)
 {
-	zend_error_handling       error_handling;
 	php_phongo_utcdatetime_t* intern;
 	char*                     serialized;
 	size_t                    serialized_len;
@@ -329,12 +284,9 @@ static PHP_METHOD(UTCDateTime, unserialize)
 
 	intern = Z_UTCDATETIME_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_STRING(serialized, serialized_len)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	PHP_VAR_UNSERIALIZE_INIT(var_hash);
 	if (!php_var_unserialize(&props, (const unsigned char**) &serialized, (unsigned char*) serialized + serialized_len, &var_hash)) {
@@ -348,20 +300,16 @@ static PHP_METHOD(UTCDateTime, unserialize)
 
 	php_phongo_utcdatetime_init_from_hash(intern, HASH_OF(&props));
 	zval_ptr_dtor(&props);
-} /* }}} */
+}
 
-/* {{{ proto array MongoDB\Driver\UTCDateTime::__serialize()
-*/
-static PHP_METHOD(UTCDateTime, __serialize)
+static PHP_METHOD(MongoDB_BSON_UTCDateTime, __serialize)
 {
 	PHONGO_PARSE_PARAMETERS_NONE();
 
 	RETURN_ARR(php_phongo_utcdatetime_get_properties_hash(PHONGO_COMPAT_OBJ_P(getThis()), true));
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\UTCDateTime::__unserialize(array $data)
-*/
-static PHP_METHOD(UTCDateTime, __unserialize)
+static PHP_METHOD(MongoDB_BSON_UTCDateTime, __unserialize)
 {
 	zval* data;
 
@@ -370,54 +318,12 @@ static PHP_METHOD(UTCDateTime, __unserialize)
 	PHONGO_PARSE_PARAMETERS_END();
 
 	php_phongo_utcdatetime_init_from_hash(Z_UTCDATETIME_OBJ_P(getThis()), Z_ARRVAL_P(data));
-} /* }}} */
+}
 
-/* {{{ MongoDB\BSON\UTCDateTime function entries */
-/* clang-format off */
-ZEND_BEGIN_ARG_INFO_EX(ai_UTCDateTime___construct, 0, 0, 0)
-	ZEND_ARG_INFO(0, milliseconds)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_UTCDateTime___set_state, 0, 0, 1)
-	ZEND_ARG_ARRAY_INFO(0, properties, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(ai_UTCDateTime___toString, 0, 0, IS_STRING, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_UTCDateTime___unserialize, 0, 0, 1)
-	ZEND_ARG_ARRAY_INFO(0, data, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_UTCDateTime_jsonSerialize, 0, 0, IS_ARRAY, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_UTCDateTime_unserialize, 0, 0, 1)
-	ZEND_ARG_INFO(0, serialized)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_UTCDateTime_void, 0, 0, 0)
-ZEND_END_ARG_INFO()
-
-static zend_function_entry php_phongo_utcdatetime_me[] = {
-	PHP_ME(UTCDateTime, __construct, ai_UTCDateTime___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(UTCDateTime, __serialize, ai_UTCDateTime_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(UTCDateTime, __set_state, ai_UTCDateTime___set_state, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-	PHP_ME(UTCDateTime, __toString, ai_UTCDateTime___toString, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(UTCDateTime, __unserialize, ai_UTCDateTime___unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(UTCDateTime, jsonSerialize, ai_UTCDateTime_jsonSerialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(UTCDateTime, serialize, ai_UTCDateTime_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(UTCDateTime, unserialize, ai_UTCDateTime_unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(UTCDateTime, toDateTime, ai_UTCDateTime_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_FE_END
-};
-/* clang-format on */
-/* }}} */
-
-/* {{{ MongoDB\BSON\UTCDateTime object handlers */
+/* MongoDB\BSON\UTCDateTime object handlers */
 static zend_object_handlers php_phongo_handler_utcdatetime;
 
-static void php_phongo_utcdatetime_free_object(zend_object* object) /* {{{ */
+static void php_phongo_utcdatetime_free_object(zend_object* object)
 {
 	php_phongo_utcdatetime_t* intern = Z_OBJ_UTCDATETIME(object);
 
@@ -427,9 +333,9 @@ static void php_phongo_utcdatetime_free_object(zend_object* object) /* {{{ */
 		zend_hash_destroy(intern->properties);
 		FREE_HASHTABLE(intern->properties);
 	}
-} /* }}} */
+}
 
-static zend_object* php_phongo_utcdatetime_create_object(zend_class_entry* class_type) /* {{{ */
+static zend_object* php_phongo_utcdatetime_create_object(zend_class_entry* class_type)
 {
 	php_phongo_utcdatetime_t* intern = zend_object_alloc(sizeof(php_phongo_utcdatetime_t), class_type);
 
@@ -439,9 +345,9 @@ static zend_object* php_phongo_utcdatetime_create_object(zend_class_entry* class
 	intern->std.handlers = &php_phongo_handler_utcdatetime;
 
 	return &intern->std;
-} /* }}} */
+}
 
-static zend_object* php_phongo_utcdatetime_clone_object(phongo_compat_object_handler_type* object) /* {{{ */
+static zend_object* php_phongo_utcdatetime_clone_object(phongo_compat_object_handler_type* object)
 {
 	php_phongo_utcdatetime_t* intern;
 	php_phongo_utcdatetime_t* new_intern;
@@ -456,9 +362,9 @@ static zend_object* php_phongo_utcdatetime_clone_object(phongo_compat_object_han
 	php_phongo_utcdatetime_init(new_intern, intern->milliseconds);
 
 	return new_object;
-} /* }}} */
+}
 
-static int php_phongo_utcdatetime_compare_objects(zval* o1, zval* o2) /* {{{ */
+static int php_phongo_utcdatetime_compare_objects(zval* o1, zval* o2)
 {
 	php_phongo_utcdatetime_t *intern1, *intern2;
 
@@ -472,33 +378,23 @@ static int php_phongo_utcdatetime_compare_objects(zval* o1, zval* o2) /* {{{ */
 	}
 
 	return 0;
-} /* }}} */
+}
 
-static HashTable* php_phongo_utcdatetime_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_utcdatetime_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp)
 {
 	*is_temp = 1;
 	return php_phongo_utcdatetime_get_properties_hash(object, true);
-} /* }}} */
+}
 
-static HashTable* php_phongo_utcdatetime_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
+static HashTable* php_phongo_utcdatetime_get_properties(phongo_compat_object_handler_type* object)
 {
 	return php_phongo_utcdatetime_get_properties_hash(object, false);
-} /* }}} */
-/* }}} */
+}
 
-void php_phongo_utcdatetime_init_ce(INIT_FUNC_ARGS) /* {{{ */
+void php_phongo_utcdatetime_init_ce(INIT_FUNC_ARGS)
 {
-	zend_class_entry ce;
-
-	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\BSON", "UTCDateTime", php_phongo_utcdatetime_me);
-	php_phongo_utcdatetime_ce                = zend_register_internal_class(&ce);
+	php_phongo_utcdatetime_ce                = register_class_MongoDB_BSON_UTCDateTime(php_phongo_utcdatetime_interface_ce, php_phongo_json_serializable_ce, php_phongo_type_ce, zend_ce_serializable);
 	php_phongo_utcdatetime_ce->create_object = php_phongo_utcdatetime_create_object;
-	PHONGO_CE_FINAL(php_phongo_utcdatetime_ce);
-
-	zend_class_implements(php_phongo_utcdatetime_ce, 1, php_phongo_utcdatetime_interface_ce);
-	zend_class_implements(php_phongo_utcdatetime_ce, 1, php_phongo_json_serializable_ce);
-	zend_class_implements(php_phongo_utcdatetime_ce, 1, php_phongo_type_ce);
-	zend_class_implements(php_phongo_utcdatetime_ce, 1, zend_ce_serializable);
 
 #if PHP_VERSION_ID >= 80000
 	zend_class_implements(php_phongo_utcdatetime_ce, 1, zend_ce_stringable);
@@ -511,4 +407,17 @@ void php_phongo_utcdatetime_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	php_phongo_handler_utcdatetime.get_properties = php_phongo_utcdatetime_get_properties;
 	php_phongo_handler_utcdatetime.free_obj       = php_phongo_utcdatetime_free_object;
 	php_phongo_handler_utcdatetime.offset         = XtOffsetOf(php_phongo_utcdatetime_t, std);
-} /* }}} */
+}
+
+bool phongo_utcdatetime_new(zval* object, int64_t msec_since_epoch)
+{
+	php_phongo_utcdatetime_t* intern;
+
+	object_init_ex(object, php_phongo_utcdatetime_ce);
+
+	intern               = Z_UTCDATETIME_OBJ_P(object);
+	intern->milliseconds = msec_since_epoch;
+	intern->initialized  = true;
+
+	return true;
+}
