@@ -25,6 +25,7 @@
 #include "php_phongo.h"
 #include "phongo_bson_encode.h"
 #include "phongo_error.h"
+#include "BulkWrite_arginfo.h"
 
 #include "MongoDB/WriteConcern.h"
 
@@ -33,15 +34,15 @@
 zend_class_entry* php_phongo_bulkwrite_ce;
 
 /* Extracts the "_id" field of a BSON document into a return value. */
-static void php_phongo_bulkwrite_extract_id(bson_t* doc, zval** return_value) /* {{{ */
+static void php_phongo_bulkwrite_extract_id(bson_t* doc, zval** return_value)
 {
 	zval*                 id = NULL;
 	php_phongo_bson_state state;
 
 	PHONGO_BSON_INIT_STATE(state);
-	state.map.root_type = PHONGO_TYPEMAP_NATIVE_ARRAY;
+	state.map.root.type = PHONGO_TYPEMAP_NATIVE_ARRAY;
 
-	if (!php_phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &state)) {
+	if (!php_phongo_bson_to_zval_ex(doc, &state)) {
 		goto cleanup;
 	}
 
@@ -53,10 +54,10 @@ static void php_phongo_bulkwrite_extract_id(bson_t* doc, zval** return_value) /*
 
 cleanup:
 	zval_ptr_dtor(&state.zchild);
-} /* }}} */
+}
 
 /* Returns whether any top-level field names in the document contain a "$". */
-static inline bool php_phongo_bulkwrite_update_has_operators(bson_t* bupdate) /* {{{ */
+static inline bool php_phongo_bulkwrite_update_has_operators(bson_t* bupdate)
 {
 	bson_iter_t iter;
 
@@ -69,10 +70,10 @@ static inline bool php_phongo_bulkwrite_update_has_operators(bson_t* bupdate) /*
 	}
 
 	return false;
-} /* }}} */
+}
 
 /* Returns whether the update document is considered an aggregation pipeline */
-static inline bool php_phongo_bulkwrite_update_is_pipeline(bson_t* bupdate) /* {{{ */
+static inline bool php_phongo_bulkwrite_update_is_pipeline(bson_t* bupdate)
 {
 	bson_iter_t iter;
 	bson_iter_t child;
@@ -113,12 +114,12 @@ static inline bool php_phongo_bulkwrite_update_is_pipeline(bson_t* bupdate) /* {
 
 	/* should return false when the document is empty */
 	return i != 0;
-} /* }}} */
+}
 
 /* Returns whether the BSON array's keys are a sequence of integer strings
  * starting with "0". BSON_APPEND_ARRAY considers it the caller's responsibility
  * to ensure that the array's keys are properly formatted. */
-static inline bool php_phongo_bulkwrite_bson_array_has_valid_keys(bson_t* array) /* {{{ */
+static inline bool php_phongo_bulkwrite_bson_array_has_valid_keys(bson_t* array)
 {
 	bson_iter_t iter;
 
@@ -142,11 +143,11 @@ static inline bool php_phongo_bulkwrite_bson_array_has_valid_keys(bson_t* array)
 	}
 
 	return true;
-} /* }}} */
+}
 
 /* Appends an array field for the given opts document and key. Returns true on
  * success; otherwise, false is returned and an exception is thrown. */
-static bool php_phongo_bulkwrite_opts_append_array(bson_t* opts, const char* key, zval* zarr) /* {{{ */
+static bool php_phongo_bulkwrite_opts_append_array(bson_t* opts, const char* key, zval* zarr)
 {
 	zval*  value = php_array_fetch(zarr, key);
 	bson_t b     = BSON_INITIALIZER;
@@ -177,11 +178,11 @@ static bool php_phongo_bulkwrite_opts_append_array(bson_t* opts, const char* key
 
 	bson_destroy(&b);
 	return true;
-} /* }}} */
+}
 
 /* Appends a document field for the given opts document and key. Returns true on
  * success; otherwise, false is returned and an exception is thrown. */
-static bool php_phongo_bulkwrite_opts_append_document(bson_t* opts, const char* key, zval* zarr) /* {{{ */
+static bool php_phongo_bulkwrite_opts_append_document(bson_t* opts, const char* key, zval* zarr)
 {
 	zval*  value = php_array_fetch(zarr, key);
 	bson_t b     = BSON_INITIALIZER;
@@ -206,7 +207,7 @@ static bool php_phongo_bulkwrite_opts_append_document(bson_t* opts, const char*
 
 	bson_destroy(&b);
 	return true;
-} /* }}} */
+}
 
 #define PHONGO_BULKWRITE_APPEND_BOOL(opt, value)                                                       \
 	if (!BSON_APPEND_BOOL(boptions, (opt), (value))) {                                                 \
@@ -239,7 +240,7 @@ static bool php_phongo_bulkwrite_opts_append_document(bson_t* opts, const char*
  *
  * The "hint" option must be a string or document. Check for both types and
  * merge into BSON options accordingly. */
-static bool php_phongo_bulkwrite_opt_hint(bson_t* boptions, zval* zoptions) /* {{{ */
+static bool php_phongo_bulkwrite_opt_hint(bson_t* boptions, zval* zoptions)
 {
 	/* The "hint" option (or "$hint" modifier) must be a string or document.
 	 * Check for both types and merge into BSON options accordingly. */
@@ -262,10 +263,10 @@ static bool php_phongo_bulkwrite_opt_hint(bson_t* boptions, zval* zoptions) /* {
 	}
 
 	return true;
-} /* }}} */
+}
 
 /* Applies options (including defaults) for an update operation. */
-static bool php_phongo_bulkwrite_update_apply_options(bson_t* boptions, zval* zoptions) /* {{{ */
+static bool php_phongo_bulkwrite_update_apply_options(bson_t* boptions, zval* zoptions)
 {
 	bool multi = false, upsert = false;
 
@@ -284,10 +285,10 @@ static bool php_phongo_bulkwrite_update_apply_options(bson_t* boptions, zval* zo
 	}
 
 	return true;
-} /* }}} */
+}
 
 /* Applies options (including defaults) for a delete operation. */
-static bool php_phongo_bulkwrite_delete_apply_options(bson_t* boptions, zval* zoptions) /* {{{ */
+static bool php_phongo_bulkwrite_delete_apply_options(bson_t* boptions, zval* zoptions)
 {
 	int32_t limit = 0;
 
@@ -303,29 +304,25 @@ static bool php_phongo_bulkwrite_delete_apply_options(bson_t* boptions, zval* zo
 	}
 
 	return true;
-} /* }}} */
+}
 
 #undef PHONGO_BULKWRITE_APPEND_BOOL
 #undef PHONGO_BULKWRITE_APPEND_INT32
 #undef PHONGO_BULKWRITE_OPT_DOCUMENT
 
-/* {{{ proto void MongoDB\Driver\BulkWrite::__construct([array $options = array()])
-   Constructs a new BulkWrite */
-static PHP_METHOD(BulkWrite, __construct)
+/* Constructs a new BulkWrite */
+static PHP_METHOD(MongoDB_Driver_BulkWrite, __construct)
 {
-	zend_error_handling     error_handling;
 	php_phongo_bulkwrite_t* intern;
 	zval*                   options = NULL;
 	zend_bool               ordered = 1;
 
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|a!", &options) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(0, 1)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_ARRAY_OR_NULL(options)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	if (options && php_array_existsc(options, "ordered")) {
 		ordered = php_array_fetchc_bool(options, "ordered");
@@ -366,7 +363,7 @@ static PHP_METHOD(BulkWrite, __construct)
 		zval* value = php_array_fetch(options, "comment");
 
 		intern->comment = ecalloc(1, sizeof(bson_value_t));
-		php_phongo_zval_to_bson_value(value, PHONGO_BSON_NONE, intern->comment);
+		phongo_zval_to_bson_value(value, intern->comment);
 
 		if (EG(exception)) {
 			/* Exception should already have been thrown */
@@ -375,13 +372,11 @@ static PHP_METHOD(BulkWrite, __construct)
 
 		mongoc_bulk_operation_set_comment(intern->bulk, intern->comment);
 	}
-} /* }}} */
+}
 
-/* {{{ proto mixed MongoDB\Driver\BulkWrite::insert(array|object $document)
-   Adds an insert operation to the BulkWrite */
-static PHP_METHOD(BulkWrite, insert)
+/* Adds an insert operation to the BulkWrite */
+static PHP_METHOD(MongoDB_Driver_BulkWrite, insert)
 {
-	zend_error_handling     error_handling;
 	php_phongo_bulkwrite_t* intern;
 	zval*                   zdocument;
 	bson_t                  bdocument = BSON_INITIALIZER, boptions = BSON_INITIALIZER;
@@ -391,12 +386,9 @@ static PHP_METHOD(BulkWrite, insert)
 
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "A", &zdocument) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	PHONGO_PARAM_ARRAY_OR_OBJECT(zdocument)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	bson_flags |= PHONGO_BSON_RETURN_ID;
 
@@ -424,13 +416,11 @@ cleanup:
 	bson_destroy(&bdocument);
 	bson_destroy(&boptions);
 	bson_clear(&bson_out);
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\BulkWrite::update(array|object $query, array|object $newObj[, array $updateOptions = array()])
-   Adds an update operation to the BulkWrite */
-static PHP_METHOD(BulkWrite, update)
+/* Adds an update operation to the BulkWrite */
+static PHP_METHOD(MongoDB_Driver_BulkWrite, update)
 {
-	zend_error_handling     error_handling;
 	php_phongo_bulkwrite_t* intern;
 	zval *                  zquery, *zupdate, *zoptions = NULL;
 	bson_t                  bquery = BSON_INITIALIZER, bupdate = BSON_INITIALIZER, boptions = BSON_INITIALIZER;
@@ -438,12 +428,12 @@ static PHP_METHOD(BulkWrite, update)
 
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "AA|a!", &zquery, &zupdate, &zoptions) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(2, 3)
+	PHONGO_PARAM_ARRAY_OR_OBJECT(zquery)
+	PHONGO_PARAM_ARRAY_OR_OBJECT(zupdate)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_ARRAY_OR_NULL(zoptions)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	php_phongo_zval_to_bson(zquery, PHONGO_BSON_NONE, &bquery, NULL);
 
@@ -491,13 +481,11 @@ cleanup:
 	bson_destroy(&bquery);
 	bson_destroy(&bupdate);
 	bson_destroy(&boptions);
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\BulkWrite::delete(array|object $query[, array $deleteOptions = array()])
-   Adds a delete operation to the BulkWrite */
-static PHP_METHOD(BulkWrite, delete)
+/* Adds a delete operation to the BulkWrite */
+static PHP_METHOD(MongoDB_Driver_BulkWrite, delete)
 {
-	zend_error_handling     error_handling;
 	php_phongo_bulkwrite_t* intern;
 	zval *                  zquery, *zoptions = NULL;
 	bson_t                  bquery = BSON_INITIALIZER, boptions = BSON_INITIALIZER;
@@ -505,12 +493,11 @@ static PHP_METHOD(BulkWrite, delete)
 
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "A|a!", &zquery, &zoptions) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 2)
+	PHONGO_PARAM_ARRAY_OR_OBJECT(zquery)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_ARRAY_OR_NULL(zoptions)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	php_phongo_zval_to_bson(zquery, PHONGO_BSON_NONE, &bquery, NULL);
 
@@ -539,70 +526,26 @@ static PHP_METHOD(BulkWrite, delete)
 cleanup:
 	bson_destroy(&bquery);
 	bson_destroy(&boptions);
-} /* }}} */
+}
 
-/* {{{ proto integer MongoDB\Driver\BulkWrite::count()
-   Returns the number of operations that have been added to the BulkWrite */
-static PHP_METHOD(BulkWrite, count)
+/* Returns the number of operations that have been added to the BulkWrite */
+static PHP_METHOD(MongoDB_Driver_BulkWrite, count)
 {
-	zend_error_handling     error_handling;
 	php_phongo_bulkwrite_t* intern;
 
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	RETURN_LONG(intern->num_ops);
-} /* }}} */
+}
 
-/* {{{ MongoDB\Driver\BulkWrite function entries */
-/* clang-format off */
-ZEND_BEGIN_ARG_INFO_EX(ai_BulkWrite___construct, 0, 0, 0)
-	ZEND_ARG_ARRAY_INFO(0, options, 1)
-ZEND_END_ARG_INFO()
+PHONGO_DISABLED_WAKEUP(MongoDB_Driver_BulkWrite)
 
-ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_BulkWrite_count, 0, 0, IS_LONG, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_BulkWrite_insert, 0, 0, 1)
-	ZEND_ARG_INFO(0, document)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_BulkWrite_update, 0, 0, 2)
-	ZEND_ARG_INFO(0, query)
-	ZEND_ARG_INFO(0, newObj)
-	ZEND_ARG_ARRAY_INFO(0, updateOptions, 1)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_BulkWrite_delete, 0, 0, 1)
-	ZEND_ARG_INFO(0, query)
-	ZEND_ARG_ARRAY_INFO(0, deleteOptions, 1)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_BulkWrite_void, 0, 0, 0)
-ZEND_END_ARG_INFO()
-
-static zend_function_entry php_phongo_bulkwrite_me[] = {
-	PHP_ME(BulkWrite, __construct, ai_BulkWrite___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(BulkWrite, insert, ai_BulkWrite_insert, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(BulkWrite, update, ai_BulkWrite_update, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(BulkWrite, delete, ai_BulkWrite_delete, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(BulkWrite, count, ai_BulkWrite_count, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_BulkWrite_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_FE_END
-};
-/* clang-format on */
-/* }}} */
-
-/* {{{ MongoDB\Driver\BulkWrite object handlers */
+/* MongoDB\Driver\BulkWrite object handlers */
 static zend_object_handlers php_phongo_handler_bulkwrite;
 
-static void php_phongo_bulkwrite_free_object(zend_object* object) /* {{{ */
+static void php_phongo_bulkwrite_free_object(zend_object* object)
 {
 	php_phongo_bulkwrite_t* intern = Z_OBJ_BULKWRITE(object);
 
@@ -632,9 +575,9 @@ static void php_phongo_bulkwrite_free_object(zend_object* object) /* {{{ */
 	if (!Z_ISUNDEF(intern->session)) {
 		zval_ptr_dtor(&intern->session);
 	}
-} /* }}} */
+}
 
-static zend_object* php_phongo_bulkwrite_create_object(zend_class_entry* class_type) /* {{{ */
+static zend_object* php_phongo_bulkwrite_create_object(zend_class_entry* class_type)
 {
 	php_phongo_bulkwrite_t* intern = zend_object_alloc(sizeof(php_phongo_bulkwrite_t), class_type);
 
@@ -644,9 +587,9 @@ static zend_object* php_phongo_bulkwrite_create_object(zend_class_entry* class_t
 	intern->std.handlers = &php_phongo_handler_bulkwrite;
 
 	return &intern->std;
-} /* }}} */
+}
 
-static HashTable* php_phongo_bulkwrite_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_bulkwrite_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp)
 {
 	zval                    retval = ZVAL_STATIC_INIT;
 	php_phongo_bulkwrite_t* intern = NULL;
@@ -678,7 +621,7 @@ static HashTable* php_phongo_bulkwrite_get_debug_info(phongo_compat_object_handl
 	if (intern->comment) {
 		zval zv;
 
-		if (!php_phongo_bson_value_to_zval(intern->comment, &zv)) {
+		if (!phongo_bson_value_to_zval_legacy(intern->comment, &zv)) {
 			zval_ptr_dtor(&zv);
 			goto done;
 		}
@@ -689,7 +632,7 @@ static HashTable* php_phongo_bulkwrite_get_debug_info(phongo_compat_object_handl
 	if (intern->let) {
 		zval zv;
 
-		if (!php_phongo_bson_to_zval(bson_get_data(intern->let), intern->let->len, &zv)) {
+		if (!php_phongo_bson_to_zval(intern->let, &zv)) {
 			zval_ptr_dtor(&zv);
 			goto done;
 		}
@@ -718,23 +661,16 @@ static HashTable* php_phongo_bulkwrite_get_debug_info(phongo_compat_object_handl
 
 done:
 	return Z_ARRVAL(retval);
-} /* }}} */
-/* }}} */
+}
 
-void php_phongo_bulkwrite_init_ce(INIT_FUNC_ARGS) /* {{{ */
+void php_phongo_bulkwrite_init_ce(INIT_FUNC_ARGS)
 {
-	zend_class_entry ce;
-
-	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "BulkWrite", php_phongo_bulkwrite_me);
-	php_phongo_bulkwrite_ce                = zend_register_internal_class(&ce);
+	php_phongo_bulkwrite_ce                = register_class_MongoDB_Driver_BulkWrite(zend_ce_countable);
 	php_phongo_bulkwrite_ce->create_object = php_phongo_bulkwrite_create_object;
-	PHONGO_CE_FINAL(php_phongo_bulkwrite_ce);
 	PHONGO_CE_DISABLE_SERIALIZATION(php_phongo_bulkwrite_ce);
 
 	memcpy(&php_phongo_handler_bulkwrite, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	php_phongo_handler_bulkwrite.get_debug_info = php_phongo_bulkwrite_get_debug_info;
 	php_phongo_handler_bulkwrite.free_obj       = php_phongo_bulkwrite_free_object;
 	php_phongo_handler_bulkwrite.offset         = XtOffsetOf(php_phongo_bulkwrite_t, std);
-
-	zend_class_implements(php_phongo_bulkwrite_ce, 1, zend_ce_countable);
-} /* }}} */
+}
