@@ -32,15 +32,11 @@
 #include "MongoDB/Server.h"
 #include "MongoDB/Session.h"
 #include "MongoDB/WriteConcern.h"
+#include "Session_arginfo.h"
+#include "BSON/Timestamp.h"
 
 zend_class_entry* php_phongo_session_ce;
 
-#define PHONGO_TRANSACTION_NONE "none"
-#define PHONGO_TRANSACTION_STARTING "starting"
-#define PHONGO_TRANSACTION_IN_PROGRESS "in_progress"
-#define PHONGO_TRANSACTION_COMMITTED "committed"
-#define PHONGO_TRANSACTION_ABORTED "aborted"
-
 #define SESSION_CHECK_LIVELINESS(i, m)                                  \
 	if (!(i)->client_session) {                                         \
 		phongo_throw_exception(                                         \
@@ -159,11 +155,12 @@ static void php_phongo_transaction_options_to_zval(mongoc_client_session_t* cs,
 	mongoc_transaction_opts_destroy(opts);
 }
 
-/* {{{ proto void MongoDB\Driver\Session::advanceClusterTime(array|object $clusterTime)
-   Advances the cluster time for this Session */
-static PHP_METHOD(Session, advanceClusterTime)
+PHONGO_DISABLED_CONSTRUCTOR(MongoDB_Driver_Session)
+PHONGO_DISABLED_WAKEUP(MongoDB_Driver_Session)
+
+/* Advances the cluster time for this Session */
+static PHP_METHOD(MongoDB_Driver_Session, advanceClusterTime)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	zval*                 zcluster_time;
 	bson_t                cluster_time = BSON_INITIALIZER;
@@ -171,12 +168,9 @@ static PHP_METHOD(Session, advanceClusterTime)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "advanceClusterTime")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "A", &zcluster_time) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	PHONGO_PARAM_ARRAY_OR_OBJECT(zcluster_time)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	php_phongo_zval_to_bson(zcluster_time, PHONGO_BSON_NONE, &cluster_time, NULL);
 
@@ -189,13 +183,11 @@ static PHP_METHOD(Session, advanceClusterTime)
 
 cleanup:
 	bson_destroy(&cluster_time);
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Session::advanceOperationTime(MongoDB\BSON\TimestampInterface $timestamp)
-   Advances the operation time for this Session */
-static PHP_METHOD(Session, advanceOperationTime)
+/* Advances the operation time for this Session */
+static PHP_METHOD(MongoDB_Driver_Session, advanceOperationTime)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	zval*                 ztimestamp;
 	uint32_t              timestamp = 0;
@@ -204,25 +196,20 @@ static PHP_METHOD(Session, advanceOperationTime)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "advanceOperationTime")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &ztimestamp, php_phongo_timestamp_interface_ce) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_OBJECT_OF_CLASS(ztimestamp, php_phongo_timestamp_interface_ce)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	if (!php_phongo_session_get_timestamp_parts(ztimestamp, &timestamp, &increment)) {
 		return;
 	}
 
 	mongoc_client_session_advance_operation_time(intern->client_session, timestamp, increment);
-} /* }}} */
+}
 
-/* {{{ proto object|null MongoDB\Driver\Session::getClusterTime()
-   Returns the cluster time for this Session */
-static PHP_METHOD(Session, getClusterTime)
+/* Returns the cluster time for this Session */
+static PHP_METHOD(MongoDB_Driver_Session, getClusterTime)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	const bson_t*         cluster_time;
 	php_phongo_bson_state state;
@@ -232,12 +219,7 @@ static PHP_METHOD(Session, getClusterTime)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getClusterTime")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	cluster_time = mongoc_client_session_get_cluster_time(intern->client_session);
 
@@ -245,20 +227,18 @@ static PHP_METHOD(Session, getClusterTime)
 		RETURN_NULL();
 	}
 
-	if (!php_phongo_bson_to_zval_ex(bson_get_data(cluster_time), cluster_time->len, &state)) {
+	if (!php_phongo_bson_to_zval_ex(cluster_time, &state)) {
 		/* Exception should already have been thrown */
 		zval_ptr_dtor(&state.zchild);
 		return;
 	}
 
 	RETURN_ZVAL(&state.zchild, 0, 1);
-} /* }}} */
+}
 
-/* {{{ proto object MongoDB\Driver\Session::getLogicalSessionId()
-   Returns the logical session ID for this Session */
-static PHP_METHOD(Session, getLogicalSessionId)
+/* Returns the logical session ID for this Session */
+static PHP_METHOD(MongoDB_Driver_Session, getLogicalSessionId)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	const bson_t*         lsid;
 	php_phongo_bson_state state;
@@ -268,41 +248,29 @@ static PHP_METHOD(Session, getLogicalSessionId)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getLogicalSessionId")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	lsid = mongoc_client_session_get_lsid(intern->client_session);
 
-	if (!php_phongo_bson_to_zval_ex(bson_get_data(lsid), lsid->len, &state)) {
+	if (!php_phongo_bson_to_zval_ex(lsid, &state)) {
 		/* Exception should already have been thrown */
 		zval_ptr_dtor(&state.zchild);
 		return;
 	}
 
 	RETURN_ZVAL(&state.zchild, 0, 1);
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\BSON\Timestamp|null MongoDB\Driver\Session::getOperationTime()
-   Returns the operation time for this Session */
-static PHP_METHOD(Session, getOperationTime)
+/* Returns the operation time for this Session */
+static PHP_METHOD(MongoDB_Driver_Session, getOperationTime)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	uint32_t              timestamp, increment;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getOperationTime")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	mongoc_client_session_get_operation_time(intern->client_session, &timestamp, &increment);
 
@@ -313,26 +281,19 @@ static PHP_METHOD(Session, getOperationTime)
 		RETURN_NULL();
 	}
 
-	php_phongo_bson_new_timestamp_from_increment_and_timestamp(return_value, increment, timestamp);
-} /* }}} */
+	phongo_timestamp_new(return_value, increment, timestamp);
+}
 
-/* {{{ proto MongoDB\Driver\Server|null MongoDB\Driver\Session::getServer()
-   Returns the server this session is pinned to */
-static PHP_METHOD(Session, getServer)
+/* Returns the server this session is pinned to */
+static PHP_METHOD(MongoDB_Driver_Session, getServer)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	uint32_t              server_id = 0;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getServer")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	server_id = mongoc_client_session_get_server_id(intern->client_session);
 
@@ -342,45 +303,31 @@ static PHP_METHOD(Session, getServer)
 	}
 
 	phongo_server_init(return_value, &intern->manager, server_id);
-} /* }}} */
+}
 
-/* {{{ proto array|null MongoDB\Driver\Session::getTransactionOptions()
-   Returns options for the currently running transaction */
-static PHP_METHOD(Session, getTransactionOptions)
+/* Returns options for the currently running transaction */
+static PHP_METHOD(MongoDB_Driver_Session, getTransactionOptions)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getTransactionOptions")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	php_phongo_transaction_options_to_zval(intern->client_session, return_value);
-} /* }}} */
+}
 
-/* {{{ proto string MongoDB\Driver\Session::getTransactionState()
-   Returns the current transaction state for this session */
-static PHP_METHOD(Session, getTransactionState)
+/* Returns the current transaction state for this session */
+static PHP_METHOD(MongoDB_Driver_Session, getTransactionState)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	const char*           state;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getTransactionState")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	state = php_phongo_get_transaction_state_string(mongoc_client_session_get_transaction_state(intern->client_session));
 	if (!state) {
@@ -389,7 +336,7 @@ static PHP_METHOD(Session, getTransactionState)
 	}
 
 	RETURN_STRING(state);
-} /* }}} */
+}
 
 /* Creates a opts structure from an array optionally containing an RP, RC,
  * WC object, and/or maxCommitTimeMS int. Returns NULL if no options were found,
@@ -488,11 +435,9 @@ mongoc_transaction_opt_t* php_mongodb_session_parse_transaction_options(zval* op
 	return opts;
 }
 
-/* {{{ proto void MongoDB\Driver\Session::startTransaction([array $options = null])
-   Starts a new transaction */
-static PHP_METHOD(Session, startTransaction)
+/* Starts a new transaction */
+static PHP_METHOD(MongoDB_Driver_Session, startTransaction)
 {
-	zend_error_handling       error_handling;
 	php_phongo_session_t*     intern;
 	zval*                     options     = NULL;
 	mongoc_transaction_opt_t* txn_options = NULL;
@@ -501,12 +446,10 @@ static PHP_METHOD(Session, startTransaction)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "startTransaction")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|a!", &options) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(0, 1)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_ARRAY_OR_NULL(options)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	if (options) {
 		txn_options = php_mongodb_session_parse_transaction_options(options);
@@ -522,13 +465,11 @@ static PHP_METHOD(Session, startTransaction)
 	if (txn_options) {
 		mongoc_transaction_opts_destroy(txn_options);
 	}
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Session::commitTransaction(void)
-   Commits an existing transaction */
-static PHP_METHOD(Session, commitTransaction)
+/* Commits an existing transaction */
+static PHP_METHOD(MongoDB_Driver_Session, commitTransaction)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	bson_error_t          error;
 	bson_t                reply;
@@ -536,68 +477,48 @@ static PHP_METHOD(Session, commitTransaction)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "commitTransaction")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	if (!mongoc_client_session_commit_transaction(intern->client_session, &reply, &error)) {
 		phongo_throw_exception_from_bson_error_t_and_reply(&error, &reply);
 	}
 
 	bson_destroy(&reply);
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Session::abortTransaction(void)
-   Aborts (rolls back) an existing transaction */
-static PHP_METHOD(Session, abortTransaction)
+/* Aborts (rolls back) an existing transaction */
+static PHP_METHOD(MongoDB_Driver_Session, abortTransaction)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	bson_error_t          error;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "abortTransaction")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	if (!mongoc_client_session_abort_transaction(intern->client_session, &error)) {
 		phongo_throw_exception_from_bson_error_t(&error);
 	}
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Session::endSession(void)
-   Ends the session, and a running transaction if active */
-static PHP_METHOD(Session, endSession)
+/* Ends the session, and a running transaction if active */
+static PHP_METHOD(MongoDB_Driver_Session, endSession)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	mongoc_client_session_destroy(intern->client_session);
 	intern->client_session = NULL;
-} /* }}} */
+}
 
-/* {{{ proto bool MongoDB\Driver\Session::isDirty()
-   Returns whether the session is dirty (i.e. was used with a command that
+/* Returns whether the session is dirty (i.e. was used with a command that
    encountered a network error) and will be discarded when returned to the
    server session pool. */
-static PHP_METHOD(Session, isDirty)
+static PHP_METHOD(MongoDB_Driver_Session, isDirty)
 {
 	php_phongo_session_t* intern = Z_SESSION_OBJ_P(getThis());
 
@@ -606,71 +527,25 @@ static PHP_METHOD(Session, isDirty)
 	PHONGO_PARSE_PARAMETERS_NONE();
 
 	RETVAL_BOOL(mongoc_client_session_get_dirty(intern->client_session));
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Session::isInTransaction(void)
-   Returns whether a multi-document transaction is in progress */
-static PHP_METHOD(Session, isInTransaction)
+/* Returns whether a multi-document transaction is in progress */
+static PHP_METHOD(MongoDB_Driver_Session, isInTransaction)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "isInTransaction")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	RETURN_BOOL(mongoc_client_session_in_transaction(intern->client_session));
-} /* }}} */
+}
 
-/* {{{ MongoDB\Driver\Session function entries */
-ZEND_BEGIN_ARG_INFO_EX(ai_Session_advanceClusterTime, 0, 0, 1)
-	ZEND_ARG_INFO(0, clusterTime)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Session_advanceOperationTime, 0, 0, 1)
-	ZEND_ARG_INFO(0, timestamp)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Session_startTransaction, 0, 0, 0)
-	ZEND_ARG_ARRAY_INFO(0, options, 1)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Session_void, 0, 0, 0)
-ZEND_END_ARG_INFO()
-
-static zend_function_entry php_phongo_session_me[] = {
-	/* clang-format off */
-	PHP_ME(Session, abortTransaction, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, advanceClusterTime, ai_Session_advanceClusterTime, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, advanceOperationTime, ai_Session_advanceOperationTime, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, commitTransaction, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, endSession, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, getClusterTime, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, getLogicalSessionId, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, getOperationTime, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, getServer, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, getTransactionOptions, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, getTransactionState, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, isDirty, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, isInTransaction, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, startTransaction, ai_Session_startTransaction, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	ZEND_NAMED_ME(__construct, PHP_FN(MongoDB_disabled___construct), ai_Session_void, ZEND_ACC_PRIVATE | ZEND_ACC_FINAL)
-	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_FE_END
-	/* clang-format on */
-};
-/* }}} */
-
-/* {{{ MongoDB\Driver\Session object handlers */
+/* MongoDB\Driver\Session object handlers */
 static zend_object_handlers php_phongo_handler_session;
 
-static void php_phongo_session_free_object(zend_object* object) /* {{{ */
+static void php_phongo_session_free_object(zend_object* object)
 {
 	php_phongo_session_t* intern = Z_OBJ_SESSION(object);
 
@@ -690,9 +565,9 @@ static void php_phongo_session_free_object(zend_object* object) /* {{{ */
 	if (!Z_ISUNDEF(intern->manager)) {
 		zval_ptr_dtor(&intern->manager);
 	}
-} /* }}} */
+}
 
-static zend_object* php_phongo_session_create_object(zend_class_entry* class_type) /* {{{ */
+static zend_object* php_phongo_session_create_object(zend_class_entry* class_type)
 {
 	php_phongo_session_t* intern = zend_object_alloc(sizeof(php_phongo_session_t), class_type);
 
@@ -704,9 +579,9 @@ static zend_object* php_phongo_session_create_object(zend_class_entry* class_typ
 	intern->std.handlers = &php_phongo_handler_session;
 
 	return &intern->std;
-} /* }}} */
+}
 
-static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp)
 {
 	php_phongo_session_t* intern = NULL;
 	zval                  retval = ZVAL_STATIC_INIT;
@@ -728,7 +603,7 @@ static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler
 
 		PHONGO_BSON_INIT_DEBUG_STATE(state);
 
-		if (!php_phongo_bson_to_zval_ex(bson_get_data(lsid), lsid->len, &state)) {
+		if (!php_phongo_bson_to_zval_ex(lsid, &state)) {
 			zval_ptr_dtor(&state.zchild);
 			goto done;
 		}
@@ -744,7 +619,7 @@ static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler
 
 			PHONGO_BSON_INIT_DEBUG_STATE(state);
 
-			if (!php_phongo_bson_to_zval_ex(bson_get_data(cluster_time), cluster_time->len, &state)) {
+			if (!php_phongo_bson_to_zval_ex(cluster_time, &state)) {
 				zval_ptr_dtor(&state.zchild);
 				goto done;
 			}
@@ -769,7 +644,11 @@ static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler
 		if (timestamp && increment) {
 			zval ztimestamp;
 
-			php_phongo_bson_new_timestamp_from_increment_and_timestamp(&ztimestamp, increment, timestamp);
+			if (!phongo_timestamp_new(&ztimestamp, increment, timestamp)) {
+				/* Exception should already have been thrown */
+				goto done;
+			}
+
 			ADD_ASSOC_ZVAL_EX(&retval, "operationTime", &ztimestamp);
 		} else {
 			ADD_ASSOC_NULL_EX(&retval, "operationTime");
@@ -812,32 +691,21 @@ static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler
 
 done:
 	return Z_ARRVAL(retval);
-} /* }}} */
-/* }}} */
+}
 
-void php_phongo_session_init_ce(INIT_FUNC_ARGS) /* {{{ */
+void php_phongo_session_init_ce(INIT_FUNC_ARGS)
 {
-	zend_class_entry ce;
-
-	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "Session", php_phongo_session_me);
-	php_phongo_session_ce                = zend_register_internal_class(&ce);
+	php_phongo_session_ce                = register_class_MongoDB_Driver_Session();
 	php_phongo_session_ce->create_object = php_phongo_session_create_object;
-	PHONGO_CE_FINAL(php_phongo_session_ce);
 	PHONGO_CE_DISABLE_SERIALIZATION(php_phongo_session_ce);
 
 	memcpy(&php_phongo_handler_session, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	php_phongo_handler_session.get_debug_info = php_phongo_session_get_debug_info;
 	php_phongo_handler_session.free_obj       = php_phongo_session_free_object;
 	php_phongo_handler_session.offset         = XtOffsetOf(php_phongo_session_t, std);
+}
 
-	zend_declare_class_constant_string(php_phongo_session_ce, ZEND_STRL("TRANSACTION_NONE"), PHONGO_TRANSACTION_NONE);
-	zend_declare_class_constant_string(php_phongo_session_ce, ZEND_STRL("TRANSACTION_STARTING"), PHONGO_TRANSACTION_STARTING);
-	zend_declare_class_constant_string(php_phongo_session_ce, ZEND_STRL("TRANSACTION_IN_PROGRESS"), PHONGO_TRANSACTION_IN_PROGRESS);
-	zend_declare_class_constant_string(php_phongo_session_ce, ZEND_STRL("TRANSACTION_COMMITTED"), PHONGO_TRANSACTION_COMMITTED);
-	zend_declare_class_constant_string(php_phongo_session_ce, ZEND_STRL("TRANSACTION_ABORTED"), PHONGO_TRANSACTION_ABORTED);
-} /* }}} */
-
-void phongo_session_init(zval* return_value, zval* manager, mongoc_client_session_t* client_session) /* {{{ */
+void phongo_session_init(zval* return_value, zval* manager, mongoc_client_session_t* client_session)
 {
 	php_phongo_session_t* session;
 
@@ -848,4 +716,3 @@ void phongo_session_init(zval* return_value, zval* manager, mongoc_client_sessio
 
 	ZVAL_ZVAL(&session->manager, manager, 1, 0);
 }
-/* }}} */
