@@ -26,12 +26,13 @@
 #include "phongo_util.h"
 
 #include "MongoDB/WriteConcern.h"
+#include "WriteConcern_arginfo.h"
 
 zend_class_entry* php_phongo_writeconcern_ce;
 
 /* Initialize the object from a HashTable and return whether it was successful.
  * An exception will be thrown on error. */
-static bool php_phongo_writeconcern_init_from_hash(php_phongo_writeconcern_t* intern, HashTable* props) /* {{{ */
+static bool php_phongo_writeconcern_init_from_hash(php_phongo_writeconcern_t* intern, HashTable* props)
 {
 	zval *w, *wtimeout, *j;
 
@@ -104,25 +105,23 @@ failure:
 	mongoc_write_concern_destroy(intern->write_concern);
 	intern->write_concern = NULL;
 	return false;
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\WriteConcern::__construct(integer|string $w[, integer $wtimeout[, boolean $journal]])
-   Constructs a new WriteConcern */
-static PHP_METHOD(WriteConcern, __construct)
+/* Constructs a new WriteConcern */
+static PHP_METHOD(MongoDB_Driver_WriteConcern, __construct)
 {
-	zend_error_handling        error_handling;
 	php_phongo_writeconcern_t* intern;
-	zval *                     w, *journal;
+	zval *                     w, *journal = NULL;
 	zend_long                  wtimeout = 0;
 
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "z|lz", &w, &wtimeout, &journal) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 3)
+	Z_PARAM_ZVAL(w)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_LONG(wtimeout)
+	Z_PARAM_ZVAL(journal)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	intern->write_concern = mongoc_write_concern_new();
 
@@ -145,7 +144,7 @@ static PHP_METHOD(WriteConcern, __construct)
 
 	switch (ZEND_NUM_ARGS()) {
 		case 3:
-			if (Z_TYPE_P(journal) != IS_NULL) {
+			if (journal && Z_TYPE_P(journal) != IS_NULL) {
 				if (zend_is_true(journal) && (mongoc_write_concern_get_w(intern->write_concern) == MONGOC_WRITE_CONCERN_W_UNACKNOWLEDGED || mongoc_write_concern_get_w(intern->write_concern) == MONGOC_WRITE_CONCERN_W_ERRORS_IGNORED)) {
 					phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Cannot enable journaling when using w = 0");
 					return;
@@ -168,23 +167,17 @@ static PHP_METHOD(WriteConcern, __construct)
 		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Write concern is not valid");
 		return;
 	}
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\Driver\WriteConcern MongoDB\Driver\WriteConcern::__set_state(array $properties)
-*/
-static PHP_METHOD(WriteConcern, __set_state)
+static PHP_METHOD(MongoDB_Driver_WriteConcern, __set_state)
 {
-	zend_error_handling        error_handling;
 	php_phongo_writeconcern_t* intern;
 	HashTable*                 props;
 	zval*                      array;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &array) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_ARRAY(array)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	object_init_ex(return_value, php_phongo_writeconcern_ce);
 
@@ -192,24 +185,17 @@ static PHP_METHOD(WriteConcern, __set_state)
 	props  = Z_ARRVAL_P(array);
 
 	php_phongo_writeconcern_init_from_hash(intern, props);
-} /* }}} */
+}
 
-/* {{{ proto string|integer|null MongoDB\Driver\WriteConcern::getW()
-   Returns the WriteConcern "w" option */
-static PHP_METHOD(WriteConcern, getW)
+/* Returns the WriteConcern "w" option */
+static PHP_METHOD(MongoDB_Driver_WriteConcern, getW)
 {
-	zend_error_handling        error_handling;
 	php_phongo_writeconcern_t* intern;
 	const char*                wtag;
 
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	wtag = mongoc_write_concern_get_wtag(intern->write_concern);
 
@@ -226,24 +212,17 @@ static PHP_METHOD(WriteConcern, getW)
 	}
 
 	RETURN_NULL();
-} /* }}} */
+}
 
-/* {{{ proto integer MongoDB\Driver\WriteConcern::getWtimeout()
-   Returns the WriteConcern "wtimeout" option */
-static PHP_METHOD(WriteConcern, getWtimeout)
+/* Returns the WriteConcern "wtimeout" option */
+static PHP_METHOD(MongoDB_Driver_WriteConcern, getWtimeout)
 {
-	zend_error_handling        error_handling;
 	php_phongo_writeconcern_t* intern;
 	int64_t                    wtimeout;
 
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	wtimeout = mongoc_write_concern_get_wtimeout_int64(intern->write_concern);
 
@@ -254,52 +233,38 @@ static PHP_METHOD(WriteConcern, getWtimeout)
 #endif
 
 	RETURN_LONG(wtimeout);
-} /* }}} */
+}
 
-/* {{{ proto null|boolean MongoDB\Driver\WriteConcern::getJournal()
-   Returns the WriteConcern "journal" option */
-static PHP_METHOD(WriteConcern, getJournal)
+/* Returns the WriteConcern "journal" option */
+static PHP_METHOD(MongoDB_Driver_WriteConcern, getJournal)
 {
-	zend_error_handling        error_handling;
 	php_phongo_writeconcern_t* intern;
 
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	if (mongoc_write_concern_journal_is_set(intern->write_concern)) {
 		RETURN_BOOL(mongoc_write_concern_get_journal(intern->write_concern));
 	}
 
 	RETURN_NULL();
-} /* }}} */
+}
 
-/* {{{ proto boolean MongoDB\Driver\WriteConcern::isDefault()
-   Returns whether the write concern has not been modified (i.e. from a Manager
+/* Returns whether the write concern has not been modified (i.e. from a Manager
    with no write concern URI options). */
-static PHP_METHOD(WriteConcern, isDefault)
+static PHP_METHOD(MongoDB_Driver_WriteConcern, isDefault)
 {
-	zend_error_handling        error_handling;
 	php_phongo_writeconcern_t* intern;
 
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	RETURN_BOOL(mongoc_write_concern_is_default(intern->write_concern));
-} /* }}} */
+}
 
-static HashTable* php_phongo_writeconcern_get_properties_hash(phongo_compat_object_handler_type* object, bool is_temp, bool is_bson, bool is_serialize) /* {{{ */
+static HashTable* php_phongo_writeconcern_get_properties_hash(phongo_compat_object_handler_type* object, bool is_temp, bool is_bson, bool is_serialize)
 {
 	php_phongo_writeconcern_t* intern;
 	HashTable*                 props;
@@ -368,30 +333,18 @@ static HashTable* php_phongo_writeconcern_get_properties_hash(phongo_compat_obje
 	}
 
 	return props;
-} /* }}} */
+}
 
-/* {{{ proto array MongoDB\Driver\WriteConcern::bsonSerialize()
-*/
-static PHP_METHOD(WriteConcern, bsonSerialize)
+static PHP_METHOD(MongoDB_Driver_WriteConcern, bsonSerialize)
 {
-	zend_error_handling error_handling;
-
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	ZVAL_ARR(return_value, php_phongo_writeconcern_get_properties_hash(PHONGO_COMPAT_OBJ_P(getThis()), true, true, false));
 	convert_to_object(return_value);
-} /* }}} */
+}
 
-/* {{{ proto string MongoDB\Driver\WriteConcern::serialize()
-*/
-static PHP_METHOD(WriteConcern, serialize)
+static PHP_METHOD(MongoDB_Driver_WriteConcern, serialize)
 {
-	zend_error_handling        error_handling;
 	php_phongo_writeconcern_t* intern;
 	zval                       retval;
 	php_serialize_data_t       var_hash;
@@ -402,12 +355,7 @@ static PHP_METHOD(WriteConcern, serialize)
 
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	if (!intern->write_concern) {
 		return;
@@ -448,13 +396,10 @@ static PHP_METHOD(WriteConcern, serialize)
 
 	smart_str_free(&buf);
 	zval_ptr_dtor(&retval);
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\WriteConcern::unserialize(string $serialized)
-*/
-static PHP_METHOD(WriteConcern, unserialize)
+static PHP_METHOD(MongoDB_Driver_WriteConcern, unserialize)
 {
-	zend_error_handling        error_handling;
 	php_phongo_writeconcern_t* intern;
 	char*                      serialized;
 	size_t                     serialized_len;
@@ -463,12 +408,9 @@ static PHP_METHOD(WriteConcern, unserialize)
 
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_STRING(serialized, serialized_len)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	if (!serialized_len) {
 		return;
@@ -486,20 +428,16 @@ static PHP_METHOD(WriteConcern, unserialize)
 
 	php_phongo_writeconcern_init_from_hash(intern, HASH_OF(&props));
 	zval_ptr_dtor(&props);
-} /* }}} */
+}
 
-/* {{{ proto array MongoDB\Driver\WriteConcern::__serialize()
-*/
-static PHP_METHOD(WriteConcern, __serialize)
+static PHP_METHOD(MongoDB_Driver_WriteConcern, __serialize)
 {
 	PHONGO_PARSE_PARAMETERS_NONE();
 
 	RETURN_ARR(php_phongo_writeconcern_get_properties_hash(PHONGO_COMPAT_OBJ_P(getThis()), true, false, true));
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\WriteConcern::__unserialize(array $data)
-*/
-static PHP_METHOD(WriteConcern, __unserialize)
+static PHP_METHOD(MongoDB_Driver_WriteConcern, __unserialize)
 {
 	zval* data;
 
@@ -508,52 +446,12 @@ static PHP_METHOD(WriteConcern, __unserialize)
 	PHONGO_PARSE_PARAMETERS_END();
 
 	php_phongo_writeconcern_init_from_hash(Z_WRITECONCERN_OBJ_P(getThis()), Z_ARRVAL_P(data));
-} /* }}} */
+}
 
-/* {{{ MongoDB\Driver\WriteConcern function entries */
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteConcern___construct, 0, 0, 1)
-	ZEND_ARG_INFO(0, w)
-	ZEND_ARG_INFO(0, wtimeout)
-	ZEND_ARG_INFO(0, journal)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteConcern___set_state, 0, 0, 1)
-	ZEND_ARG_ARRAY_INFO(0, properties, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteConcern___unserialize, 0, 0, 1)
-	ZEND_ARG_ARRAY_INFO(0, data, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteConcern_unserialize, 0, 0, 1)
-	ZEND_ARG_INFO(0, serialized)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteConcern_void, 0, 0, 0)
-ZEND_END_ARG_INFO()
-
-static zend_function_entry php_phongo_writeconcern_me[] = {
-	/* clang-format off */
-	PHP_ME(WriteConcern, __construct, ai_WriteConcern___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(WriteConcern, __serialize, ai_WriteConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(WriteConcern, __set_state, ai_WriteConcern___set_state, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-	PHP_ME(WriteConcern, __unserialize, ai_WriteConcern___unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(WriteConcern, getW, ai_WriteConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(WriteConcern, getWtimeout, ai_WriteConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(WriteConcern, getJournal, ai_WriteConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(WriteConcern, isDefault, ai_WriteConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(WriteConcern, bsonSerialize, ai_WriteConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(WriteConcern, serialize, ai_WriteConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(WriteConcern, unserialize, ai_WriteConcern_unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_FE_END
-	/* clang-format on */
-};
-/* }}} */
-
-/* {{{ MongoDB\Driver\WriteConcern object handlers */
+/* MongoDB\Driver\WriteConcern object handlers */
 static zend_object_handlers php_phongo_handler_writeconcern;
 
-static void php_phongo_writeconcern_free_object(zend_object* object) /* {{{ */
+static void php_phongo_writeconcern_free_object(zend_object* object)
 {
 	php_phongo_writeconcern_t* intern = Z_OBJ_WRITECONCERN(object);
 
@@ -567,9 +465,9 @@ static void php_phongo_writeconcern_free_object(zend_object* object) /* {{{ */
 	if (intern->write_concern) {
 		mongoc_write_concern_destroy(intern->write_concern);
 	}
-} /* }}} */
+}
 
-static zend_object* php_phongo_writeconcern_create_object(zend_class_entry* class_type) /* {{{ */
+static zend_object* php_phongo_writeconcern_create_object(zend_class_entry* class_type)
 {
 	php_phongo_writeconcern_t* intern = zend_object_alloc(sizeof(php_phongo_writeconcern_t), class_type);
 
@@ -579,41 +477,32 @@ static zend_object* php_phongo_writeconcern_create_object(zend_class_entry* clas
 	intern->std.handlers = &php_phongo_handler_writeconcern;
 
 	return &intern->std;
-} /* }}} */
+}
 
-static HashTable* php_phongo_writeconcern_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_writeconcern_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp)
 {
 	*is_temp = 1;
 	return php_phongo_writeconcern_get_properties_hash(object, true, false, false);
-} /* }}} */
+}
 
-static HashTable* php_phongo_writeconcern_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
+static HashTable* php_phongo_writeconcern_get_properties(phongo_compat_object_handler_type* object)
 {
 	return php_phongo_writeconcern_get_properties_hash(object, false, false, false);
-} /* }}} */
+}
 
-void php_phongo_writeconcern_init_ce(INIT_FUNC_ARGS) /* {{{ */
+void php_phongo_writeconcern_init_ce(INIT_FUNC_ARGS)
 {
-	zend_class_entry ce;
-
-	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "WriteConcern", php_phongo_writeconcern_me);
-	php_phongo_writeconcern_ce                = zend_register_internal_class(&ce);
+	php_phongo_writeconcern_ce                = register_class_MongoDB_Driver_WriteConcern(php_phongo_serializable_ce, zend_ce_serializable);
 	php_phongo_writeconcern_ce->create_object = php_phongo_writeconcern_create_object;
-	PHONGO_CE_FINAL(php_phongo_writeconcern_ce);
-
-	zend_class_implements(php_phongo_writeconcern_ce, 1, php_phongo_serializable_ce);
-	zend_class_implements(php_phongo_writeconcern_ce, 1, zend_ce_serializable);
 
 	memcpy(&php_phongo_handler_writeconcern, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	php_phongo_handler_writeconcern.get_debug_info = php_phongo_writeconcern_get_debug_info;
 	php_phongo_handler_writeconcern.get_properties = php_phongo_writeconcern_get_properties;
 	php_phongo_handler_writeconcern.free_obj       = php_phongo_writeconcern_free_object;
 	php_phongo_handler_writeconcern.offset         = XtOffsetOf(php_phongo_writeconcern_t, std);
+}
 
-	zend_declare_class_constant_stringl(php_phongo_writeconcern_ce, ZEND_STRL("MAJORITY"), ZEND_STRL(PHONGO_WRITE_CONCERN_W_MAJORITY));
-} /* }}} */
-
-void phongo_writeconcern_init(zval* return_value, const mongoc_write_concern_t* write_concern) /* {{{ */
+void phongo_writeconcern_init(zval* return_value, const mongoc_write_concern_t* write_concern)
 {
 	php_phongo_writeconcern_t* intern;
 
@@ -622,9 +511,8 @@ void phongo_writeconcern_init(zval* return_value, const mongoc_write_concern_t*
 	intern                = Z_WRITECONCERN_OBJ_P(return_value);
 	intern->write_concern = mongoc_write_concern_copy(write_concern);
 }
-/* }}} */
 
-const mongoc_write_concern_t* phongo_write_concern_from_zval(zval* zwrite_concern) /* {{{ */
+const mongoc_write_concern_t* phongo_write_concern_from_zval(zval* zwrite_concern)
 {
 	if (zwrite_concern) {
 		php_phongo_writeconcern_t* intern = Z_WRITECONCERN_OBJ_P(zwrite_concern);
@@ -635,9 +523,9 @@ const mongoc_write_concern_t* phongo_write_concern_from_zval(zval* zwrite_concer
 	}
 
 	return NULL;
-} /* }}} */
+}
 
-void php_phongo_write_concern_to_zval(zval* retval, const mongoc_write_concern_t* write_concern) /* {{{ */
+void php_phongo_write_concern_to_zval(zval* retval, const mongoc_write_concern_t* write_concern)
 {
 	const char*   wtag     = mongoc_write_concern_get_wtag(write_concern);
 	const int32_t w        = mongoc_write_concern_get_w(write_concern);
@@ -668,4 +556,4 @@ void php_phongo_write_concern_to_zval(zval* retval, const mongoc_write_concern_t
 		ADD_ASSOC_LONG_EX(retval, "wtimeout", wtimeout);
 #endif
 	}
-} /* }}} */
+}
