@@ -129,6 +129,11 @@ _make_command (mongoc_change_stream_t *stream, bson_t *command)
    if (stream->full_document_before_change) {
       bson_concat (&change_stream_doc, stream->full_document_before_change);
    }
+   if (stream->show_expanded_events) {
+      BSON_APPEND_BOOL (&change_stream_doc,
+                        "showExpandedEvents",
+                        stream->show_expanded_events);
+   }
 
    if (stream->resumed) {
       /* Change stream spec: Resume Process */
@@ -289,13 +294,8 @@ _make_cursor (mongoc_change_stream_t *stream)
       goto cleanup;
    }
 
-   server_stream =
-      mongoc_cluster_stream_for_reads (&stream->client->cluster,
-                                       stream->read_prefs,
-                                       cs,
-                                       &reply,
-                                       /* Not aggregate-with-write */ false,
-                                       &stream->err);
+   server_stream = mongoc_cluster_stream_for_reads (
+      &stream->client->cluster, stream->read_prefs, cs, &reply, &stream->err);
    if (!server_stream) {
       bson_destroy (&stream->err_doc);
       bson_copy_to (&reply, &stream->err_doc);
@@ -433,6 +433,7 @@ _change_stream_init (mongoc_change_stream_t *stream,
 
    stream->batch_size = stream->opts.batchSize;
    stream->max_await_time_ms = stream->opts.maxAwaitTimeMS;
+   stream->show_expanded_events = stream->opts.showExpandedEvents;
 
    /* Accept two forms of user pipeline:
     * 1. A document like: { "pipeline": [...] }
@@ -470,8 +471,7 @@ _mongoc_change_stream_new_from_collection (const mongoc_collection_t *coll,
    mongoc_change_stream_t *stream;
    BSON_ASSERT (coll);
 
-   stream =
-      (mongoc_change_stream_t *) bson_malloc0 (sizeof (mongoc_change_stream_t));
+   stream = BSON_ALIGNED_ALLOC0 (mongoc_change_stream_t);
    stream->db = bson_strdup (coll->db);
    stream->coll = bson_strdup (coll->collection);
    stream->read_prefs = mongoc_read_prefs_copy (coll->read_prefs);
@@ -490,8 +490,7 @@ _mongoc_change_stream_new_from_database (const mongoc_database_t *db,
    mongoc_change_stream_t *stream;
    BSON_ASSERT (db);
 
-   stream =
-      (mongoc_change_stream_t *) bson_malloc0 (sizeof (mongoc_change_stream_t));
+   stream = BSON_ALIGNED_ALLOC0 (mongoc_change_stream_t);
    stream->db = bson_strdup (db->name);
    stream->coll = NULL;
    stream->read_prefs = mongoc_read_prefs_copy (db->read_prefs);
@@ -510,8 +509,7 @@ _mongoc_change_stream_new_from_client (mongoc_client_t *client,
    mongoc_change_stream_t *stream;
    BSON_ASSERT (client);
 
-   stream =
-      (mongoc_change_stream_t *) bson_malloc0 (sizeof (mongoc_change_stream_t));
+   stream = BSON_ALIGNED_ALLOC0 (mongoc_change_stream_t);
    stream->db = bson_strdup ("admin");
    stream->coll = NULL;
    stream->read_prefs = mongoc_read_prefs_copy (client->read_prefs);
