@@ -17,6 +17,9 @@
 #include "bson/bson.h"
 
 #include <php.h>
+#if PHP_VERSION_ID >= 80100
+#include <Zend/zend_enum.h>
+#endif
 #include <Zend/zend_interfaces.h>
 
 #include "php_phongo.h"
@@ -43,12 +46,13 @@
 #endif
 
 /* Forwards declarations */
+static void php_phongo_bson_append(bson_t* bson, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, const char* key, long key_len, zval* entry);
 static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, bson_t* bson, bson_t** bson_out);
 
 /* Determines whether the argument should be serialized as a BSON array or
  * document. IS_ARRAY is returned if the argument's keys are a sequence of
  * integers starting at zero; otherwise, IS_OBJECT is returned. */
-static int php_phongo_is_array_or_document(zval* val) /* {{{ */
+static int php_phongo_is_array_or_document(zval* val)
 {
 	HashTable* ht_data = HASH_OF(val);
 	int        count;
@@ -80,20 +84,37 @@ static int php_phongo_is_array_or_document(zval* val) /* {{{ */
 	}
 
 	return IS_ARRAY;
-} /* }}} */
+}
 
-/* Appends the array or object argument to the BSON document. If the object is
- * an instance of MongoDB\BSON\Serializable, the return value of bsonSerialize()
- * will be appended as an embedded document. Other MongoDB\BSON\Type instances
- * will be appended as the appropriate BSON type. Other array or object values
- * will be appended as an embedded document. */
-static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, const char* key, long key_len, zval* object) /* {{{ */
+/* Appends the array or object argument to the BSON document.
+ *
+ * For instances of MongoDB\BSON\Document, raw BSON data is appended as document.
+ * For instances of MongoDB\BSON\PackedArray, raw BSON data is appended as array.
+ * For instances of MongoDB\BSON\Serializable, the return value of bsonSerialize()
+ * will be appended as an embedded document.
+ * Other MongoDB\BSON\Type instances will be appended as the appropriate BSON
+ * type.
+ * Other array or object values will be appended as an embedded document.
+ */
+static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, const char* key, long key_len, zval* object)
 {
 	if (Z_TYPE_P(object) == IS_OBJECT && instanceof_function(Z_OBJCE_P(object), php_phongo_cursorid_ce)) {
 		bson_append_int64(bson, key, key_len, Z_CURSORID_OBJ_P(object)->id);
 		return;
 	}
 
+	if (Z_TYPE_P(object) == IS_OBJECT && (instanceof_function(Z_OBJCE_P(object), php_phongo_document_ce) || instanceof_function(Z_OBJCE_P(object), php_phongo_packedarray_ce))) {
+		if (instanceof_function(Z_OBJCE_P(object), php_phongo_document_ce)) {
+			php_phongo_document_t* intern = Z_DOCUMENT_OBJ_P(object);
+			bson_append_document(bson, key, key_len, intern->bson);
+		} else {
+			php_phongo_packedarray_t* intern = Z_PACKEDARRAY_OBJ_P(object);
+			bson_append_array(bson, key, key_len, intern->bson);
+		}
+
+		return;
+	}
+
 	if (Z_TYPE_P(object) == IS_OBJECT && instanceof_function(Z_OBJCE_P(object), php_phongo_type_ce)) {
 		if (instanceof_function(Z_OBJCE_P(object), php_phongo_serializable_ce)) {
 			zval   obj_data;
@@ -236,7 +257,23 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 
 		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Unexpected %s instance: %s", ZSTR_VAL(php_phongo_type_ce->name), ZSTR_VAL(Z_OBJCE_P(object)->name));
 		return;
-	} else {
+	}
+
+#if PHP_VERSION_ID >= 80100
+	if (Z_TYPE_P(object) == IS_OBJECT && Z_OBJCE_P(object)->ce_flags & ZEND_ACC_ENUM) {
+		if (Z_OBJCE_P(object)->enum_backing_type == IS_UNDEF) {
+			char* path_string = php_phongo_field_path_as_string(field_path);
+			phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Non-backed enum %s cannot be serialized for field path \"%s\"", ZSTR_VAL(Z_OBJCE_P(object)->name), path_string);
+			efree(path_string);
+			return;
+		}
+
+		php_phongo_bson_append(bson, field_path, flags, key, key_len, zend_enum_fetch_case_value(Z_OBJ_P(object)));
+		return;
+	}
+#endif /* PHP_VERSION_ID >= 80100 */
+
+	{
 		bson_t child;
 
 		mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding document");
@@ -244,12 +281,12 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 		php_phongo_zval_to_bson_internal(object, field_path, flags, &child, NULL);
 		bson_append_document_end(bson, &child);
 	}
-} /* }}} */
+}
 
 /* Appends the zval argument to the BSON document. If the argument is an object,
  * or an array that should be serialized as an embedded document, this function
  * will defer to php_phongo_bson_append_object(). */
-static void php_phongo_bson_append(bson_t* bson, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, const char* key, long key_len, zval* entry) /* {{{ */
+static void php_phongo_bson_append(bson_t* bson, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, const char* key, long key_len, zval* entry)
 {
 	php_phongo_field_path_write_item_at_current_level(field_path, key);
 
@@ -318,7 +355,12 @@ try_again:
 				break;
 			}
 
-			php_phongo_field_path_write_type_at_current_level(field_path, PHONGO_FIELD_PATH_ITEM_DOCUMENT);
+			if (Z_TYPE_P(entry) == IS_OBJECT && instanceof_function(Z_OBJCE_P(entry), php_phongo_packedarray_ce)) {
+				php_phongo_field_path_write_type_at_current_level(field_path, PHONGO_FIELD_PATH_ITEM_ARRAY);
+			} else {
+				php_phongo_field_path_write_type_at_current_level(field_path, PHONGO_FIELD_PATH_ITEM_DOCUMENT);
+			}
+
 			field_path->size++;
 			php_phongo_bson_append_object(bson, field_path, flags, key, key_len, entry);
 			field_path->size--;
@@ -337,9 +379,27 @@ try_again:
 			efree(path_string);
 		}
 	}
-} /* }}} */
+}
 
-static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, bson_t* bson, bson_t** bson_out) /* {{{ */
+/* This is based on bson_copy_to_excluding_noinit() and is necessary because
+ * bson_copy_to() cannot be used with a bson_t allocated with bson_new(). */
+static void phongo_bson_copy_to_noinit(const bson_t* src, bson_t* dst)
+{
+	bson_iter_t iter;
+
+	if (bson_iter_init(&iter, src)) {
+		while (bson_iter_next(&iter)) {
+			if (!bson_append_iter(dst, NULL, 0, &iter)) {
+				/* This should not be able to happen since we are copying from
+				 * within a valid bson_t. */
+				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Error copying \"%s\" field from source document", bson_iter_key(&iter));
+				return;
+			}
+		}
+	}
+}
+
+static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, bson_t* bson, bson_t** bson_out)
 {
 	HashTable* ht_data = NULL;
 	zval       obj_data;
@@ -357,6 +417,31 @@ static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path*
 
 	switch (Z_TYPE_P(data)) {
 		case IS_OBJECT:
+			/* Short-circuit MongoDB\BSON\Document and MongoDB\BSON\PackedArray instances - copy the data */
+			if (instanceof_function(Z_OBJCE_P(data), php_phongo_document_ce)) {
+				php_phongo_document_t* intern = Z_DOCUMENT_OBJ_P(data);
+				bson_iter_t            iter;
+
+				phongo_bson_copy_to_noinit(intern->bson, bson);
+
+				// Check if the document instance already has an _id field
+				if (flags & PHONGO_BSON_ADD_ID && bson_iter_init_find(&iter, bson, "_id")) {
+					flags &= ~PHONGO_BSON_ADD_ID;
+				}
+
+				goto done;
+			}
+
+			if (instanceof_function(Z_OBJCE_P(data), php_phongo_packedarray_ce)) {
+				php_phongo_packedarray_t* intern = Z_PACKEDARRAY_OBJ_P(data);
+
+				phongo_bson_copy_to_noinit(intern->bson, bson);
+
+				goto done;
+			}
+
+			/* For any MongoDB\BSON\Serializable, invoke the bsonSerialize method
+			 * and work with the result. */
 			if (instanceof_function(Z_OBJCE_P(data), php_phongo_serializable_ce)) {
 				zend_call_method_with_0_params(PHONGO_COMPAT_OBJ_P(data), NULL, NULL, BSON_SERIALIZE_FUNC_NAME, &obj_data);
 
@@ -389,6 +474,16 @@ static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path*
 				break;
 			}
 
+			/* For the error handling that follows, we can safely assume that we
+			 * are at the root level, since php_phongo_bson_append_object would
+			 * have already been called for a non-root level. */
+#if PHP_VERSION_ID >= 80100
+			if (Z_OBJCE_P(data)->ce_flags & ZEND_ACC_ENUM) {
+				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Enum %s cannot be serialized as a root element", ZSTR_VAL(Z_OBJCE_P(data)->name));
+				return;
+			}
+#endif /* PHP_VERSION_ID >= 80100 */
+
 			if (instanceof_function(Z_OBJCE_P(data), php_phongo_type_ce)) {
 				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "%s instance %s cannot be serialized as a root element", ZSTR_VAL(php_phongo_type_ce->name), ZSTR_VAL(Z_OBJCE_P(data)->name));
 				return;
@@ -452,6 +547,7 @@ static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path*
 		ZEND_HASH_FOREACH_END();
 	}
 
+done:
 	if (flags & PHONGO_BSON_ADD_ID) {
 		bson_oid_t oid;
 
@@ -478,24 +574,21 @@ cleanup:
 	if (!Z_ISUNDEF(obj_data)) {
 		zval_ptr_dtor(&obj_data);
 	}
-} /* }}} */
+}
 
 /* Converts the array or object argument to a BSON document. If the object is an
  * instance of MongoDB\BSON\Serializable, the return value of bsonSerialize()
  * will be used. */
-void php_phongo_zval_to_bson(zval* data, php_phongo_bson_flags_t flags, bson_t* bson, bson_t** bson_out) /* {{{ */
+void php_phongo_zval_to_bson(zval* data, php_phongo_bson_flags_t flags, bson_t* bson, bson_t** bson_out)
 {
 	php_phongo_field_path* field_path = php_phongo_field_path_alloc(false);
 
 	php_phongo_zval_to_bson_internal(data, field_path, flags, bson, bson_out);
 
 	php_phongo_field_path_free(field_path);
-} /* }}} */
+}
 
-/* Converts the argument to a bson_value_t. If the object is an instance of
- * MongoDB\BSON\Serializable, the return value of bsonSerialize() will be
- * used. It is the caller's responsibility to call bson_value_destroy. */
-void php_phongo_zval_to_bson_value(zval* data, php_phongo_bson_flags_t flags, bson_value_t* value) /* {{{ */
+static void phongo_zval_to_bson_value_ex(zval* data, php_phongo_bson_flags_t flags, bson_value_t* value)
 {
 	bson_iter_t iter;
 	bson_t      bson = BSON_INITIALIZER;
@@ -514,4 +607,71 @@ void php_phongo_zval_to_bson_value(zval* data, php_phongo_bson_flags_t flags, bs
 
 	bson_destroy(&bson);
 	zval_ptr_dtor(&data_object);
-} /* }}} */
+}
+
+/* Converts the argument to a bson_value_t. If the object is an instance of
+ * MongoDB\BSON\Serializable, the return value of bsonSerialize() will be
+ * used. It is the caller's responsibility to call bson_value_destroy.
+ *
+ * On success, the zval will be populated and true will be returned. On error,
+ * an exception will have been thrown and false will be returned. */
+bool phongo_zval_to_bson_value(zval* data, bson_value_t* value)
+{
+	zend_long lvalue;
+
+	ZVAL_DEREF(data);
+
+	switch (Z_TYPE_P(data)) {
+		case IS_UNDEF:
+		case IS_NULL:
+			value->value_type = BSON_TYPE_NULL;
+			return true;
+
+		case IS_FALSE:
+			value->value_type   = BSON_TYPE_BOOL;
+			value->value.v_bool = false;
+			return true;
+
+		case IS_TRUE:
+			value->value_type   = BSON_TYPE_BOOL;
+			value->value.v_bool = true;
+			return true;
+
+		case IS_LONG:
+			lvalue = Z_LVAL_P(data);
+
+			if (lvalue > INT32_MAX || lvalue < INT32_MIN) {
+				value->value_type    = BSON_TYPE_INT64;
+				value->value.v_int64 = lvalue;
+			} else {
+				value->value_type    = BSON_TYPE_INT32;
+				value->value.v_int32 = (int32_t) lvalue;
+			}
+
+			return true;
+
+		case IS_DOUBLE:
+			value->value_type     = BSON_TYPE_DOUBLE;
+			value->value.v_double = Z_DVAL_P(data);
+			return true;
+
+		case IS_STRING:
+			value->value_type       = BSON_TYPE_UTF8;
+			value->value.v_utf8.len = Z_STRLEN_P(data);
+
+			/* Duplicate string as bson_value_t is expected to own values */
+			value->value.v_utf8.str = bson_malloc(value->value.v_utf8.len + 1);
+			memcpy(value->value.v_utf8.str, Z_STRVAL_P(data), value->value.v_utf8.len);
+			value->value.v_utf8.str[value->value.v_utf8.len] = '\0';
+			return true;
+
+		case IS_ARRAY:
+		case IS_OBJECT:
+			/* Use php_phongo_zval_to_bson internally to convert arrays and documents */
+			phongo_zval_to_bson_value_ex(data, PHONGO_BSON_NONE, value);
+			return true;
+	}
+
+	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Unsupported type %s", zend_zval_type_name(data));
+	return false;
+}
