@@ -41,6 +41,7 @@ typedef struct {
 typedef struct {
 	mongoc_client_encryption_t* client_encryption;
 	zval                        key_vault_client_manager;
+	char*                       key_vault_namespace;
 	zend_object                 std;
 } php_phongo_clientencryption_t;
 
@@ -176,6 +177,29 @@ typedef struct {
 	zend_object std;
 } php_phongo_binary_t;
 
+typedef struct {
+	bson_t*     bson;
+	HashTable*  properties;
+	zend_object std;
+} php_phongo_packedarray_t;
+
+typedef struct {
+	bson_t*     bson;
+	HashTable*  properties;
+	zend_object std;
+} php_phongo_document_t;
+
+typedef struct {
+	zval        bson;
+	bson_iter_t iter;
+	bool        valid;
+	bool        is_array;
+	size_t      key;
+	zval        current;
+	HashTable*  properties;
+	zend_object std;
+} php_phongo_iterator_t;
+
 typedef struct {
 	char*       ref;
 	size_t      ref_len;
@@ -260,14 +284,14 @@ typedef struct {
 	zval        manager;
 	char*       command_name;
 	uint32_t    server_id;
-	uint64_t    operation_id;
-	uint64_t    request_id;
-	uint64_t    duration_micros;
+	int64_t     operation_id;
+	int64_t     request_id;
+	int64_t     duration_micros;
 	bson_t*     reply;
 	zval        z_error;
 	bool        has_service_id;
 	bson_oid_t  service_id;
-	int32_t     server_connection_id;
+	int64_t     server_connection_id;
 	zend_object std;
 } php_phongo_commandfailedevent_t;
 
@@ -275,13 +299,13 @@ typedef struct {
 	zval        manager;
 	char*       command_name;
 	uint32_t    server_id;
-	uint64_t    operation_id;
-	uint64_t    request_id;
+	int64_t     operation_id;
+	int64_t     request_id;
 	bson_t*     command;
 	char*       database_name;
 	bool        has_service_id;
 	bson_oid_t  service_id;
-	int32_t     server_connection_id;
+	int64_t     server_connection_id;
 	zend_object std;
 } php_phongo_commandstartedevent_t;
 
@@ -289,13 +313,13 @@ typedef struct {
 	zval        manager;
 	char*       command_name;
 	uint32_t    server_id;
-	uint64_t    operation_id;
-	uint64_t    request_id;
-	uint64_t    duration_micros;
+	int64_t     operation_id;
+	int64_t     request_id;
+	int64_t     duration_micros;
 	bson_t*     reply;
 	bool        has_service_id;
 	bson_oid_t  service_id;
-	int32_t     server_connection_id;
+	int64_t     server_connection_id;
 	zend_object std;
 } php_phongo_commandsucceededevent_t;
 
